import asyncio
import os
import random
from datetime import date

from fastapi import WebSocket

from ..db.utils import get_all_editors, get_all_meeting_points
from ..services.search_meeting_points import search_close_meeting_points


async def search_slots(
    longitude: float,
    latitude: float,
    start_date: date,
    end_date: date,
    radius_km: int,
    reason: str = "CNI",
    documents_number: int = 1,
    websocket: WebSocket = None,
    department_code: str = None,
):
    """function will call 'search_slots_in_editor' that intercat with editors

    Args:
        longitude (float): longitude of the user
        latitude (float): latitude of the user
        start_date (date): the first date choosed by the user
        end_date (date): the end date choosed by the user
        radius_km (int): _description_
        reason (str, optional): _description_. Defaults to "CNI".
        documents_number (int, optional): _description_. Defaults to 1.
        websocket (WebSocket, optional): _description_. Defaults to None.

    Returns:
        [Meeting_point]: all the meeting point of all the editors
        [Errors]: all the errors sendit by the editor.
    """
    all_points = get_all_meeting_points()

    if department_code:
        if department_code == "2A" or department_code == "2B":
            department_code = "20"
        all_points = [
            point
            for point in all_points
            if point["zip_code"][: len(department_code)] == department_code
        ]

    meeting_points = search_close_meeting_points(
        all_points, latitude, longitude, radius_km
    )

    all_editors_meeting_points = []
    all_editors_errors = []
    no_response_score = random.randint(1, 3)
    if (
        os.environ.get("MOCK_EMPTY_RESPONSE") in ["True", True]
    ) and no_response_score < 2:
        await asyncio.sleep(3)
    else:
        editor_futures = []
        for editor in get_all_editors():
            editor_futures.append(
                asyncio.ensure_future(
                    editor.search_slots_in_editor(
                        meeting_points,
                        start_date,
                        end_date,
                        reason,
                        documents_number,
                        websocket,
                    )
                )
            )
        all_results = await asyncio.gather(*editor_futures)
        for result in all_results:
            all_editors_meeting_points += result[0]
            if result[1]:
                all_editors_errors.append(result[1])
    return all_editors_meeting_points, all_editors_errors



from datetime import date
from typing import Any, List

from fastapi import APIRouter, HTTPException, Query, Request
from pydantic import Required
from slowapi import Limiter
from slowapi.util import get_remote_address

from ...models.municipality import MunicipalityWithSlots
from ...services.search_time_slots import search_slots

router = APIRouter()

limiter = Limiter(key_func=get_remote_address)


@router.get(
    "/SlotsFromPosition",
    response_model=List[MunicipalityWithSlots],
    responses={
        200: {
            "description": "Des rendez-vous pour les coordonnées GPS et dates fournis ont bien été trouvés.",
            "content": {
                "application/json": {
                    "example": [
                        {
                            "id": "1213",
                            "name": "Mairie ANNEXE LILLE-SECLIN",
                            "longitude": 3.0348016639327,
                            "latitude": 50.549140395451,
                            "public_entry_address": "89 RUE ROGER BOUVRY",
                            "zip_code": "59113",
                            "city_name": "SECLIN",
                            "website": "https://www.ville-seclin.fr",
                            "city_logo": "https://www.ville-seclin.fr/images/logo-ville-seclin/logo_ville_de_seclin.png",
                            "available_slots": [
                                {
                                    "datetime": "2022-12-19T10:00Z",
                                    "callback_url": "https://www.ville-seclin.fr/rendez-vous/passeports?date=2022-12-19T10:00Z",
                                },
                                {
                                    "datetime": "2022-12-19T10:20Z",
                                    "callback_url": "https://www.ville-seclin.fr/rendez-vous/passeports?date=2022-12-19T10:20Z",
                                },
                                {
                                    "datetime": "2022-12-19T10:40Z",
                                    "callback_url": "https://www.ville-seclin.fr/rendez-vous/passeports?date=2022-12-19T10:40Z",
                                },
                                {
                                    "datetime": "2022-12-19T11:00Z",
                                    "callback_url": "https://www.ville-seclin.fr/rendez-vous/passeports?date=2022-12-19T11:00Z",
                                },
                            ],
                        },
                        {
                            "id": "456456",
                            "name": "Mairie de Quartier de Lille-Sud",
                            "longitude": 3.0475818403133,
                            "latitude": 50.612875943839,
                            "public_entry_address": "83 Rue du Faubourg des Postes",
                            "zip_code": "59000",
                            "city_name": "LILLE-SECLIN",
                            "website": "https://www.lille.fr/Lille-Sud2/Mairie-de-quartier-de-Lille-Sud",
                            "city_logo": "https://www.ville-seclin.fr/images/logo-ville-seclin/logo_ville_de_seclin.png",
                            "available_slots": [
                                {
                                    "datetime": "2022-12-19T10:00Z",
                                    "callback_url": "https://www.lille.fr/Lille-Sud2/Mairie-de-quartier-de-Lille-Sud/rendez-vous/passeports?date=2022-12-19T10:00Z",
                                },
                                {
                                    "datetime": "2022-12-19T10:20Z",
                                    "callback_url": "https://www.lille.fr/Lille-Sud2/Mairie-de-quartier-de-Lille-Sud/rendez-vous/passeports?date=2022-12-19T10:20Z",
                                },
                                {
                                    "datetime": "2022-12-19T10:40Z",
                                    "callback_url": "https://www.lille.fr/Lille-Sud2/Mairie-de-quartier-de-Lille-Sud/rendez-vous/passeports?date=2022-12-19T10:40Z",
                                },
                                {
                                    "datetime": "2022-12-19T11:00Z",
                                    "callback_url": "https://www.lille.fr/Lille-Sud2/Mairie-de-quartier-de-Lille-Sud/rendez-vous/passeports?date=2022-12-19T11:00Z",
                                },
                            ],
                        },
                    ]
                }
            },
        },
        403: {
            "description": "La valeur radius_km n'est pas bonne",
            "content": {
                "application/json": {
                    "example": {"details": "radius_km value not allowed"}
                }
            },
        },
    },
)
@limiter.limit("30/minute")
async def slots_from_position(
    request: Request,
    longitude: float = Query(default=Required, example=2.352222),
    latitude: float = Query(default=Required, example=48.856613),
    start_date: date = Query(default=Required, example="2022-11-01"),
    end_date: date = Query(default=Required, example="2022-11-30"),
    radius_km: int = Query(default=40, enum=[0.99, 5, 10, 15, 20, 25, 30]),
    reason: str = Query(default="CNI", enum=["CNI", "PASSPORT", "CNI-PASSPORT"]),
    documents_number: int = Query(default=1),
) -> Any:
    """
    Recherche de rendez-vous disponible pour une position GPS
    """

    if radius_km not in [0.99, 5, 10, 15, 20, 25, 30]:
        raise HTTPException(status_code=403, detail="radius_km value not allowed")
    result, errors = await search_slots(
        longitude, latitude, start_date, end_date, radius_km, reason, documents_number
    )
    return result



from typing import Any, Dict, Generic, List, Optional, Type, TypeVar, Union

from fastapi.encoders import jsonable_encoder
from pydantic import BaseModel
from sqlalchemy.orm import Session

from ..db.postgresdb import Base

ModelType = TypeVar("ModelType", bound=Base)
CreateSchemaType = TypeVar("CreateSchemaType", bound=BaseModel)
UpdateSchemaType = TypeVar("UpdateSchemaType", bound=BaseModel)


class CRUDBase(Generic[ModelType, CreateSchemaType, UpdateSchemaType]):
    """
    CRUD object with default methods to Create, Read, Update, Delete (CRUD).

    **Parameters**

    * `model`: A SQLAlchemy model class
    * `schema`: A Pydantic model (schema) class
    """

    def __init__(self, model: Type[ModelType]):
        self.model = model

    def get(self, db: Session, id: Any) -> Optional[ModelType]:
        """
        Get a single record by ID.

        **Parameters**

        * `db`: SQLAlchemy database session
        * `id`: ID of the record to retrieve

        **Returns**

        * The record with the specified ID, or None if not found
        """
        return db.query(self.model).filter(self.model.id == id).first()

    def get_multi(
        self, db: Session, *, skip: int = 0, limit: int = 100
    ) -> List[ModelType]:
        """
        Get multiple records with optional pagination.

        **Parameters**

        * `db`: SQLAlchemy database session
        * `skip`: Number of records to skip (for pagination)
        * `limit`: Maximum number of records to retrieve (for pagination)

        **Returns**

        * A list of records matching the query
        """
        return db.query(self.model).offset(skip).limit(limit).all()

    def create(self, db: Session, *, obj_in: CreateSchemaType) -> ModelType:
        """
        Create a new record.

        **Parameters**

        * `db`: SQLAlchemy database session
        * `obj_in`: Pydantic model representing the record to create

        **Returns**

        * The newly created record
        """
        obj_in_data = jsonable_encoder(obj_in)
        db_obj = self.model(**obj_in_data)  # type: ignore
        db.add(db_obj)
        db.commit()
        db.refresh(db_obj)
        return db_obj

    def update(
        self,
        db: Session,
        *,
        db_obj: ModelType,
        obj_in: Union[UpdateSchemaType, Dict[str, Any]],
    ) -> ModelType:
        """
        Update an existing record.

        **Parameters**

        * `db`: SQLAlchemy database session
        * `db_obj`: The record to update
        * `obj_in`: Pydantic model representing the updated record

        **Returns**

        * The updated record
        """
        obj_data = jsonable_encoder(db_obj)
        if isinstance(obj_in, dict):
            update_data = obj_in
        else:
            update_data = obj_in.dict(exclude_unset=True)
        for field in obj_data:
            if field in update_data:
                setattr(db_obj, field, update_data[field])
        db.add(db_obj)
        db.commit()
        db.refresh(db_obj)
        return db_obj

    def remove(self, db: Session, *, id: int) -> ModelType:
        """
        Delete a record by ID.

        **Parameters**

        * `db`: SQLAlchemy database session
        * `id`: ID of the record to delete

        **Returns**

        * The deleted record
        """
        obj = db.query(self.model).get(id)
        db.delete(obj)
        db.commit()
        return obj



from typing import List

from sqlalchemy.orm import Session

from ..crud.base import CRUDBase
from ..models.meeting_point import MeetingPoint
from ..schemas.meeting_point import MeetingPointCreate, MeetingPointUpdate


class CRUDMeetingPoint(CRUDBase[MeetingPoint, MeetingPointCreate, MeetingPointUpdate]):
    """
    CRUD operations for the MeetingPoint model.
    """

    def get_by_ugf(self, db: Session, *, ugf: str) -> List[MeetingPoint]:
        """
        Retrieve an meeting point by ugf.

        Args:
            db (Session): The database session.
            name (str): The ugf of the meeting point to retrieve.

        Returns:
            Optional[Editor]: The editor with the given name, or None if not found.
        """
        return db.query(MeetingPoint).filter(MeetingPoint.ugf == ugf).all()

    def get_all(self, db: Session) -> List[MeetingPoint]:
        """
        Retrieve all MeetingPoint.

        Args:
            db (Session): The database session.

        Returns:
            List[MeetingPoint]: A list of all MeetingPoint.
        """
        return db.query(MeetingPoint).all()

    def update(
        self, db: Session, *, db_obj: MeetingPoint, obj_in: MeetingPoint
    ) -> MeetingPoint:
        """
        Update a MeetingPoint.
        """
        db_obj.ugf = obj_in.ugf
        db_obj.editor_name_and_id = obj_in.editor_name_and_id
        db_obj.city_name = obj_in.city_name
        db.commit()
        db.refresh(db_obj)
        return db_obj

    def save_or_update(
        self, db: Session, *, obj_in: MeetingPoint
    ) -> tuple[str, MeetingPoint]:
        """
        Save or update a MeetingPoint.
        """
        meeting_point = (
            db.query(MeetingPoint)
            .filter(
                MeetingPoint.ugf == obj_in.ugf,
                MeetingPoint.editor_name_and_id == obj_in.editor_name_and_id,
            )
            .first()
        )
        if meeting_point:
            return "unchanged", obj_in
        else:
            self.create(db, obj_in=obj_in)
        return "created", obj_in

    def get_by_editor_name_and_id(
        self, db: Session, *, editor_name_and_id: str
    ) -> MeetingPoint:
        """
        Retrieve an meeting point by editor_name_and_id.

        Args:
            db (Session): The database session.
            name (str): The editor_name_and_id of the meeting point to retrieve.

        Returns:
            Optional[Editor]: The editor with the given name, or None if not found.
        """
        return (
            db.query(MeetingPoint)
            .filter(MeetingPoint.editor_name_and_id == editor_name_and_id)
            .first()
        )


meetingPoint = CRUDMeetingPoint(MeetingPoint)



from pydantic import BaseModel


class MeetingPointBase(BaseModel):
    """
    MeetingPoint editor schema.
    """

    ugf: str
    editor_name_and_id: str
    id_editor: int
    city_name: str


class MeetingPointCreate(MeetingPointBase):
    """
    MeetingPoint create schema.
    """


class MeetingPointUpdate(MeetingPointBase):
    """
    MeetingPoint update schema.
    """


class MeetingPoint(MeetingPointBase):
    """
    MeetingPoint schema.
    """

    class Config:
        orm_mode = True



import datetime
from typing import TYPE_CHECKING

from sqlalchemy import Column, Date, ForeignKey, Integer, String
from sqlalchemy.orm import relationship

from ..db.postgresdb import Base

if TYPE_CHECKING:
    from .editor_model import Editor  # noqa: F401


class TokenAntsToEditor(Base):
    """
    Represents a token used to authenticate an editor in the ANTS system.
    """

    __tablename__ = "token_ants_to_editor"

    id = Column(Integer, primary_key=True, index=True)
    id_editor = Column(Integer, ForeignKey("editor.id"))
    token = Column(String, unique=True, nullable=False)
    validity_date = Column(Date, nullable=False)
    editor = relationship("Editor", back_populates="tokens_ants_to_editor")

    def is_valid(self):
        """
        Returns True if the token is still valid, False otherwise.
        """
        return (
            self.validity_date is None or self.validity_date > datetime.datetime.now()
        )



import os
from pathlib import Path
from typing import List

from geopy import distance

from ..core.config import get_settings
from ..models.editor import Editor
from ..models.municipality import Municipality, OfflineMunicipality
from ..services.excel_helper import read_password_protected_excel


def get_all_editors() -> List[Editor]:
    """
    Get global editors from the settings object
    """
    editors_stored: List[Editor] = get_settings().editors_list.copy()
    return editors_stored


def set_all_editors(editors: List[Editor]):
    """
    Set global editors list in the settings object
    """
    get_settings().editors_list = editors


def get_all_meeting_points() -> List[Municipality]:
    """
    Get global meeting point from the settings object
    """
    meeting_point_stored: List[Municipality] = get_settings().meeting_point_list.copy()
    return meeting_point_stored


def set_all_meeting_points(meeting_points: List[Municipality]):
    """
    Set global meeting points list in the settings object
    """
    get_settings().meeting_point_list = meeting_points

    if os.environ.get("EXCEL_PASSWORD"):
        offline_meeting_points = read_offline_meeting_points_file(meeting_points)
        set_all_offline_meeting_points(offline_meeting_points)


def get_ws_use_rates(ip_address) -> int:
    """
    Get global WebSocket use rates from the settings object for a specific ip address
    """
    if ip_address in get_settings().ws_use_rates:
        return get_settings().ws_use_rates[ip_address]
    else:
        return 0


def add_ws_use_rates(ip_address):
    """
    Add 1 to global websocket use rates list in the settings object for a specific ip address
    """
    if ip_address not in get_settings().ws_use_rates:
        get_settings().ws_use_rates[ip_address] = 0
    get_settings().ws_use_rates[ip_address] += 1


def reset_all_ws_use_rates():
    """
    Reset global websocket use rates list in the settings object
    """
    get_settings().ws_use_rates = {}


def read_offline_meeting_points_file(
    online_meeting_points,
) -> List[OfflineMunicipality]:
    """
    Read meeting points file and decrypt it
    """
    decrypted_offline_meeting_points = []
    folder_path = Path(__file__).parent
    file_rel_path = folder_path / Path("offline_meeting_points.xlsx")
    workbook = read_password_protected_excel(file_rel_path)
    main_sheet = workbook.worksheets[0]
    max_row = main_sheet.max_row

    meeting_point_id = 5000
    for i in range(2, max_row + 1):
        # if main_sheet.cell(row=i, column=1).value:
        if (
            main_sheet.cell(row=i, column=7).value
            and main_sheet.cell(row=i, column=8).value
        ):
            try:
                decrypted_offline_meeting_points.append(
                    {
                        "id": str(meeting_point_id),
                        "municipality": main_sheet.cell(row=i, column=2).value,
                        "longitude": float(main_sheet.cell(row=i, column=7).value),
                        "latitude": float(main_sheet.cell(row=i, column=8).value),
                        "public_entry_address": (
                            main_sheet.cell(row=i, column=3).value or ""
                        )
                        + (main_sheet.cell(row=i, column=4).value or ""),
                        "zip_code": main_sheet.cell(row=i, column=5).value,
                        "decoded_city_name": main_sheet.cell(row=i, column=6).value,
                        "website": main_sheet.cell(row=i, column=11).value,
                        "logo": None,
                        "phone_number": main_sheet.cell(row=i, column=9).value,
                    }
                )
            except Exception:
                pass
        meeting_point_id += 1
    # else:
    #    break

    filtered_offline_meeting_points = []
    for offline_meeting_point in decrypted_offline_meeting_points:
        is_online = False
        for online_meeting_point in online_meeting_points:
            if (
                str(offline_meeting_point["zip_code"]).strip()
                == online_meeting_point["zip_code"].strip()
            ):
                if (
                    offline_meeting_point["municipality"].strip().upper()
                    == online_meeting_point["name"].strip().upper()
                ) or (
                    offline_meeting_point["public_entry_address"].strip().upper()
                    == online_meeting_point["public_entry_address"].strip().upper()
                ):
                    is_online = True
                    print(
                        online_meeting_point["name"]
                        + " / "
                        + offline_meeting_point["municipality"]
                    )
                    break
                compare_distance = round(
                    distance.distance(
                        (
                            offline_meeting_point["latitude"],
                            offline_meeting_point["longitude"],
                        ),
                        (
                            online_meeting_point["latitude"],
                            online_meeting_point["longitude"],
                        ),
                    ).m,
                    2,
                )
                if compare_distance < 250:
                    is_online = True
                    print(
                        online_meeting_point["name"]
                        + " / "
                        + offline_meeting_point["municipality"]
                    )
                    break

        if not is_online:
            filtered_offline_meeting_points.append(offline_meeting_point)
    print("------- Done loading offline meeting points -------")
    return filtered_offline_meeting_points


def get_all_offline_meeting_points() -> List[OfflineMunicipality]:
    """
    Get global offline meeting points from the settings object
    """
    offline_meeting_point_stored: List[
        OfflineMunicipality
    ] = get_settings().offline_meeting_point_list.copy()
    return offline_meeting_point_stored


def set_all_offline_meeting_points(meeting_points: List[OfflineMunicipality]):
    """
    Set global offline meeting points list in the settings object
    """
    get_settings().offline_meeting_point_list = meeting_points



from .postgresdb import session


def create_tables():
    # check_database()
    # Base.metadata.drop_all(bind=engine)
    # print("created tables...")
    # Base.metadata.create_all(bind=engine)
    # print(Base.metadata.tables.keys())
    pass


def get_database():
    db = session()
    try:
        yield db
    finally:
        db.close()



import datetime

from pydantic import BaseModel


class TokenAntsToEditorBase(BaseModel):
    """
    Base schema for token_ants_to_editor table.
    """

    id_editor: int
    token: str
    validity_date: datetime.datetime


class TokenAntsToEditorCreate(TokenAntsToEditorBase):
    """
    Schema for creating a new token_ants_to_editor record.
    """


class TokenAntsToEditorUpdate(TokenAntsToEditorBase):
    """
    Schema for updating an existing token_ants_to_editor record.
    """


class TokenAntsToEditor(TokenAntsToEditorBase):
    """
    Schema for token_ants_to_editor table.
    """

    id: int

    class Config:
        orm_mode = True



from typing import List, Optional

from sqlalchemy.orm import Session

from ..crud.base import CRUDBase
from ..models.editor_model import Editor
from ..schemas.editor import EditorCreate, EditorUpdate


class CRUDEditor(CRUDBase[Editor, EditorCreate, EditorUpdate]):
    """
    CRUD operations for the Editor model.
    """

    def get_by_name(self, db: Session, *, name: str) -> Optional[Editor]:
        """
        Retrieve an editor by name.

        Args:
            db (Session): The database session.
            name (str): The name of the editor to retrieve.

        Returns:
            Optional[Editor]: The editor with the given name, or None if not found.
        """
        return db.query(Editor).filter(Editor.name == name).first()

    def get_all(self, db: Session) -> List[Editor]:
        """
        Retrieve all editors.

        Args:
            db (Session): The database session.

        Returns:
            List[Editor]: A list of all editors.
        """
        return db.query(Editor).all()

    def get_by_token(self, db: Session, *, token: str) -> Optional[Editor]:
        """
        Retrieve an editor by token.

        Args:
            db (Session): The database session.
            token (str): The token of the editor to retrieve.

        Returns:
            Optional[Editor]: The editor with the given token, or None if not found.
        """
        return (
            db.query(Editor)
            .filter(Editor.tokens_editor_to_ants.any(token=token))
            .first()
        )

    def get_editors_not_only_opti(self, db: Session) -> List[Editor]:
        """
        Retrieve a list of editors that are not only connected to anti-duplication.

        Args:
            db (Session): The database session.

        Returns:
            List[Editor]: A list of Editor objects.
        """
        return db.query(Editor).filter_by(only_connect_to_anti_duplication=False).all()


editor = CRUDEditor(Editor)



from pydantic import BaseModel, Field


class CertifiedMunicipalityBase(BaseModel):
    """
    Certified Municipality  schema.
    """

    ugf: str = Field(..., max_length=5)
    town_hall_name: str = Field(..., max_length=255)
    address: str
    zip_code: str = Field(..., max_length=5)
    city_name: str = Field(..., max_length=100)
    phone_number: str
    website: str = Field(..., max_length=255)
    appointment_details: str
    service_opening_date: str
    label: str = Field(..., max_length=255)

    def __hash__(self):
        return hash(
            (
                self.ugf,
                self.town_hall_name,
                self.address,
                self.zip_code,
                self.city_name,
                self.phone_number,
                self.website,
                self.appointment_details,
                self.service_opening_date,
                self.label,
            )
        )


class CertifiedMunicipalityCreate(CertifiedMunicipalityBase):
    """
    Certified Municipality create schema.
    """


class CertifiedMunicipalityUpdate(CertifiedMunicipalityBase):
    """
    Certified Municipality update schema.
    """


class CertifiedMunicipality(CertifiedMunicipalityBase):
    """
    Certified Municipality schema.
    """

    class Config:
        orm_mode = True



import re

from unidecode import unidecode


def cannonize(input_str):
    """
    Remove non-alphanumeric characters from the input string, convert it to lowercase, and replace accented characters with their non-accented equivalents.

    Args:
        input_str (str): The input string to be cannonized.

    Returns:
        str: The cannonized string with non-alphanumeric characters removed, converted to lowercase, and accented characters replaced with their non-accented equivalents.
    """
    if input_str:
        # Remove non-alphanumeric characters and convert to lowercase
        cannonized_str = re.sub(r"\W", "", input_str.lower(), flags=re.UNICODE)
        # Replace accented characters with their non-accented equivalents
        cannonized_str = unidecode(cannonized_str)
        return cannonized_str
    return ""



from pydantic import BaseModel


class EditorBase(BaseModel):
    """
    Base editor schema.
    """

    name: str
    url: str
    header_name: str


class EditorCreate(EditorBase):
    """
    Editor create schema.
    """


class EditorUpdate(EditorBase):
    """
    Editor update schema.
    """


class Editor(EditorBase):
    """
    Editor schema.
    """

    id: int

    class Config:
        orm_mode = True



import json
import os

from fastapi import Header, HTTPException, Request


async def verify_auth_token(request: Request, x_hub_rdv_auth_token: str = Header(None)):
    """
    Verify the authentication token provided in the X-HUB-RDV-AUTH-TOKEN header.

    Args:
        request (Request): The incoming request object.
        x_hub_rdv_auth_token (str, optional): The value of the X-HUB-RDV-AUTH-TOKEN header. Defaults to None.

    Raises:
        HTTPException: If the X-HUB-RDV-AUTH-TOKEN header is invalid.

    """
    if os.environ.get("ALLOW_ORIGINS"):
        origin_url = dict(request.scope["headers"]).get(b"origin", b"").decode()
        if origin_url and origin_url in os.environ.get("ALLOW_ORIGINS"):
            return
    if x_hub_rdv_auth_token not in json.loads(os.environ.get("AUTH_TOKENS")):
        raise HTTPException(
            status_code=401, detail="X-HUB-RDV-AUTH-TOKEN header invalid"
        )


async def verify_internal_auth_token(
    request: Request, x_hub_rdv_internal_auth_token: str = Header(None)
):
    """
    Verify the internal authentication token provided in the X-HUB-RDV-INTERNAL-AUTH-TOKEN header.

    Args:
        request (Request): The incoming request object.
        x_hub_rdv_internal_auth_token (str, optional): The value of the X-HUB-RDV-INTERNAL-AUTH-TOKEN header. Defaults to None.

    Raises:
        HTTPException: If the X-HUB-RDV-INTERNAL-AUTH-TOKEN header is invalid.

    """
    if os.environ.get("ADMIN_AUTH_TOKENS"):
        if x_hub_rdv_internal_auth_token in json.loads(
            os.environ.get("ADMIN_AUTH_TOKENS")
        ):
            request.state.editor_name = "RDV-DATA-ADMIN"
        else:
            raise HTTPException(
                status_code=401, detail="X-HUB-RDV-INTERNAL-AUTH-TOKEN header invalid"
            )



import logging
import os
from datetime import datetime, timedelta

import httpx

from ..logging.app_logger import write_external_service_data


async def search_appointments_in_optimisation_api(application_ids):
    _logger = logging.getLogger("root")
    optimisation_api_url = os.environ.get("OPTIMISATION_API_URL")
    optimisation_api_token = os.environ.get("OPTIMISATION_API_TOKEN")

    result = {}
    try:
        headers = {"x-rdv-opt-auth-token": optimisation_api_token}
        parameters = {"application_ids": application_ids}
        async with httpx.AsyncClient(verify=False) as async_client:
            response = await async_client.get(
                f"{optimisation_api_url}/api/status",
                headers=headers,
                params=parameters,
                timeout=15,
                follow_redirects=True,
            )
            write_external_service_data(_logger, response, editor_name=None)
            if response.status_code in [200]:
                json_response = response.json()
                for key in json_response:
                    if json_response[key] and json_response[key]["appointments"]:
                        result[key] = []
                        for appointment in json_response[key]["appointments"]:
                            result[key].append(
                                {
                                    "meeting_point": appointment["meeting_point"],
                                    "datetime": appointment["appointment_date"],
                                    "management_url": appointment["management_url"],
                                }
                            )
            else:
                raise Exception(
                    f"Status code not handled : {response.status_code} : {response.reason_phrase}"
                )
    except Exception as search_appointments_e:
        _logger.error(
            "Error while searching appointments in the optimisation api : %s",
            str(search_appointments_e),
            extra={"extra_info": {"type": "app"}},
        )
    return result


async def update_appointment_in_optimisation_api(application_id, appointment):
    _logger = logging.getLogger("root")
    optimisation_api_url = os.environ.get("OPTIMISATION_API_URL")
    optimisation_api_token = os.environ.get("OPTIMISATION_API_TOKEN")

    try:
        headers = {"x-rdv-opt-auth-token": optimisation_api_token}
        parameters = {
            "application_id": application_id,
            "meeting_point": appointment["meeting_point"],
            "appointment_date": appointment["datetime"],
            "management_url": appointment["management_url"],
        }
        async with httpx.AsyncClient(verify=False) as async_client:
            response = await async_client.post(
                f"{optimisation_api_url}/api/appointments",
                headers=headers,
                params=parameters,
                timeout=15,
                follow_redirects=True,
            )
            write_external_service_data(_logger, response, editor_name=None)
            if response.status_code in [200]:
                _logger.info(
                    "Appointment successfully created in optimisation api : %s",
                    str(appointment),
                )
            else:
                raise Exception(
                    f"Status code not handled : {response.status_code} : {response.reason_phrase}"
                )
    except Exception as create_appointment_e:
        _logger.error(
            "Error while creating appointment in the optimisation api : %s",
            str(create_appointment_e),
            extra={"extra_info": {"type": "app"}},
        )


def is_same_appointment(appointment_1, appointment_2):
    datetime_1 = datetime_from_str(appointment_1["datetime"])
    datetime_2 = datetime_from_str(appointment_2["datetime"])

    return (
        (appointment_1["meeting_point"] == appointment_2["meeting_point"])
        and (datetime_1 == datetime_2)
        and (appointment_1["management_url"] == appointment_2["management_url"])
    )


def datetime_from_str(datetime_str):
    datetime_datetime = None
    for datetime_format in [
        "%Y-%m-%dT%H:%MZ",
        "%Y-%m-%dT%H:%M:%S%z",
        "%Y-%m-%dT%H:%M:%S",
    ]:
        try:
            datetime_datetime = datetime.strptime(datetime_str, datetime_format)
            if datetime_datetime.tzinfo:
                utcoffset = datetime_datetime.utcoffset().total_seconds() / 60 / 60
                datetime_datetime = datetime_datetime.replace(tzinfo=None) + timedelta(
                    hours=utcoffset
                )
            break
        except ValueError:
            pass

    return datetime_datetime



import asyncio
import logging

from unidecode import unidecode

from ..db.utils import get_all_editors, set_all_meeting_points


async def run_daily_scheduled_functions():
    """function that collecte meeting point from editor"""
    _logger = logging.getLogger("root")
    try:
        meeting_point_list = []
        editor_futures = []
        for editor in get_all_editors():
            editor_futures.append(
                asyncio.ensure_future(editor.get_managed_meeting_points())
            )
        all_results = await asyncio.gather(*editor_futures)

        for editor_result in all_results:
            meeting_point_list += editor_result
        point_index = 201
        for point in meeting_point_list:
            point["id"] = str(point_index)
            point["decoded_city_name"] = (
                unidecode(point["city_name"])
                .replace(" ", "-")
                .replace("'", "-")
                .lower()
            )
            point_index += 1
        set_all_meeting_points(meeting_point_list)
    except Exception as daily_sch_e:
        _logger.error("Error while running daily scheduled functions : %s", daily_sch_e)



import asyncio
import logging
import os
from datetime import date
from typing import Any, List

from fastapi import APIRouter, Depends, HTTPException, Query, Request
from pydantic import Required, constr
from slowapi import Limiter
from slowapi.util import get_remote_address

from ...db.utils import get_all_editors, get_all_meeting_points
from ...models.municipality import Municipality
from ...services.mock_data import get_mock_slots
from ...services.search_meetings import (
    is_same_appointment,
    search_appointments_in_optimisation_api,
    update_appointment_in_optimisation_api,
)
from ..dependencies.auth_token import verify_auth_token

router = APIRouter()

limiter = Limiter(key_func=get_remote_address)


@router.get(
    "/getManagedMeetingPoints",
    response_model=List[Municipality],
    responses={
        200: {
            "description": "Récupère la liste des meeting points",
            "content": {
                "application/json": {
                    "example": [
                        {
                            "id": "201",
                            "name": "Mairie ANNEXE LILLE-SECLIN",
                            "longitude": 3.0348016639327,
                            "latitude": 50.549140395451,
                            "public_entry_address": "89 RUE ROGER BOUVRY",
                            "zip_code": "59113",
                            "city_name": "SECLIN",
                            "website": "https://www.ville-seclin.fr",
                            "city_logo": "https://www.ville-seclin.fr/images/logo-ville-seclin/logo_ville_de_seclin.png",
                        }
                    ]
                }
            },
        },
        "401": {
            "description": "Utilisateur non autorisé",
            "content": {
                "application/json": {"example": "X-HUB-RDV-AUTH-TOKEN header invalid"}
            },
        },
    },
    dependencies=[Depends(verify_auth_token)],
)
@limiter.limit("30/minute")
def get_managed_meeting_points(
    request: Request,
) -> Any:
    """
    Permet de récupérer la liste des meeting points.
    """

    return get_all_meeting_points()


@router.get(
    "/availableTimeSlots",
    responses={
        200: {
            "description": "Les rendez-vous disponibles pour les meeting points id ont bien été trouvés",
            "content": {
                "application/json": {
                    "example": {
                        "506": [
                            {
                                "datetime": "2022-12-19T10:00Z",
                                "callback_url": "https://www.ville-seclin.fr/rendez-vous/passeports?date=2022-12-19T10:00Z",
                            },
                            {
                                "datetime": "2022-12-19T10:20Z",
                                "callback_url": "https://www.ville-seclin.fr/rendez-vous/passeports?date=2022-12-19T10:20Z",
                            },
                            {
                                "datetime": "2022-12-19T10:40Z",
                                "callback_url": "https://www.ville-seclin.fr/rendez-vous/passeports?date=2022-12-19T10:40Z",
                            },
                            {
                                "datetime": "2022-12-19T11:00Z",
                                "callback_url": "https://www.ville-seclin.fr/rendez-vous/passeports?date=2022-12-19T11:00Z",
                            },
                        ],
                        "7789": [
                            {
                                "datetime": "2022-12-19T10:00Z",
                                "callback_url": "https://www.lille.fr/Lille-Sud2/Mairie-de-quartier-de-Lille-Sud/rendez-vous/passeports?date=2022-12-19T10:00Z",
                            },
                            {
                                "datetime": "2022-12-19T10:20Z",
                                "callback_url": "https://www.lille.fr/Lille-Sud2/Mairie-de-quartier-de-Lille-Sud/rendez-vous/passeports?date=2022-12-19T10:20Z",
                            },
                            {
                                "datetime": "2022-12-19T10:40Z",
                                "callback_url": "https://www.lille.fr/Lille-Sud2/Mairie-de-quartier-de-Lille-Sud/rendez-vous/passeports?date=2022-12-19T10:40Z",
                            },
                            {
                                "datetime": "2022-12-19T11:00Z",
                                "callback_url": "https://www.lille.fr/Lille-Sud2/Mairie-de-quartier-de-Lille-Sud/rendez-vous/passeports?date=2022-12-19T11:00Z",
                            },
                        ],
                    }
                }
            },
        },
        "401": {
            "description": "Utilisateur non autorisé",
            "content": {
                "application/json": {"example": "X-HUB-RDV-AUTH-TOKEN header invalid"}
            },
        },
        "404": {
            "description": "Meeting point id non trouvé",
            "content": {
                "application/json": {"example": "Unknown Meeting Point ID : 11TEST2111"}
            },
        },
    },
    dependencies=[Depends(verify_auth_token)],
)
@limiter.limit("30/minute")
async def get_available_time_slots(
    request: Request,
    meeting_point_ids: list[str] = Query(default=Required, example=["201", "203"]),
    start_date: date = Query(default=Required, example="2022-11-01"),
    end_date: date = Query(default=Required, example="2022-11-30"),
    reason: str = Query(default="CNI", enum=["CNI", "PASSPORT", "CNI-PASSPORT"]),
    documents_number: int = Query(default=1),
):
    """
    Récupère les rendez-vous disponibles sur une période donnée pour une liste de meeting point id.
    <br>
    Attention: L'heure qui sera affichée est l'heure exacte que l'on trouve dans la réponse.
    """

    all_points = get_all_meeting_points()

    result = {}

    meeting_points = []
    for meeting_point_id in meeting_point_ids:
        meeting_point = None
        index = 0
        while index < len(all_points):
            if meeting_point_id == all_points[index]["id"]:
                meeting_point = all_points[index]
                break
            index += 1

        if not meeting_point:
            raise HTTPException(
                status_code=404, detail=f"Unknown Meeting Point ID : {meeting_point_id}"
            )
        meeting_points.append(meeting_point)

    if os.environ.get("MOCK_EDITORS") in ["True", True]:
        for meeting_point in meeting_points:
            result[meeting_point["id"]] = get_mock_slots(
                meeting_point, start_date, end_date
            )
    else:
        editor_futures = []
        for editor in get_all_editors():
            editor_futures.append(
                asyncio.ensure_future(
                    editor.search_slots_in_editor(
                        meeting_points,
                        start_date,
                        end_date,
                        reason,
                        documents_number,
                        None,
                    )
                )
            )
        all_editor_time_slots = await asyncio.gather(*editor_futures)
        for editor_time_slots in all_editor_time_slots:
            for meeting_point_slots in editor_time_slots[0]:
                result[meeting_point_slots["id"]] = meeting_point_slots[
                    "available_slots"
                ]

    return result


@router.get(
    "/searchApplicationIds",
    responses={
        200: {
            "description": "La recherche des numéros de pré-demande a bien aboutie!",
            "content": {
                "application/json": {
                    "example": {
                        "6123155111": [
                            {
                                "meeting_point": "Mairie ANNEXE LILLE-SECLIN",
                                "datetime": "2022-12-19T10:00Z",
                                "management_url": "https://www.ville-seclin.fr/rendez-vous/predemande?num=6123155111",
                                "cancel_url": "https://www.ville-seclin.fr/rendez-vous/annulation?num=6123155111",
                            }
                        ]
                    }
                }
            },
        },
        "401": {
            "description": "Utilisateur non autorisé",
            "content": {
                "application/json": {"example": "X-HUB-RDV-AUTH-TOKEN header invalid"}
            },
        },
        "422": {
            "description": "Validation Error",
            "content": {
                "application/json": {
                    "schema": {"$ref": "#/components/schemas/HTTPValidationError"}
                }
            },
        },
    },
    dependencies=[Depends(verify_auth_token)],
)
@limiter.limit("50/minute")
async def search_application_ids(
    request: Request,
    application_ids: List[
        constr(regex=r"^([a-zA-Z0-9]{10}[,;:\-/.\s])*[a-zA-Z0-9]{10}$")
    ] = Query(default=Required, example=["6123155111", "6123155222"]),
) -> Any:
    """
    Récupère la liste des meeting points liée aux numéros de pré-demande donnés.
    """
    _logger = logging.getLogger("root")

    editor_futures = []
    for editor in get_all_editors():
        editor_futures.append(
            asyncio.ensure_future(editor.search_meetings(application_ids))
        )
    try:
        meetings = await asyncio.gather(*editor_futures)

        searchApplicationId_result = {}
        for editor_meetings in meetings:
            for key in editor_meetings:
                if editor_meetings[key]:
                    for _ in editor_meetings[key]:
                        if "cancel_url" in _:
                            del _["cancel_url"]

                    if key not in searchApplicationId_result:
                        searchApplicationId_result[key] = []

                    searchApplicationId_result[key] += editor_meetings[key]

        result = await search_appointments_in_optimisation_api(application_ids)
        for application_id, appointments in searchApplicationId_result.items():
            if application_id not in result:
                result[application_id] = []
            for appointment in appointments:
                already_found = False
                for opti_appointment in result[application_id]:
                    if is_same_appointment(appointment, opti_appointment):
                        already_found = True
                        break
                if not already_found:
                    result[application_id].append(appointment)
                    try:
                        await update_appointment_in_optimisation_api(
                            application_id, appointment
                        )
                        _logger.info(
                            "Data regularisation on the optimisation api : %s, %s",
                            application_id,
                            appointment,
                        )
                    except Exception as update_appointment_in_opt_exc:
                        _logger.error(
                            "Error while updating appointment in optimisation api : %s",
                            update_appointment_in_opt_exc,
                        )
                        _logger.info(
                            "Appointment : %s, %s",
                            application_id,
                            appointment,
                        )

        return result
    except Exception as search_exc:
        _logger.error("Error during search application Ids : %s", search_exc)
        raise HTTPException(status_code=500, detail="Internal server error")



import logging
from typing import List

from geopy import distance
from starlette.exceptions import HTTPException
from starlette.status import HTTP_400_BAD_REQUEST

from ..models.municipality import (
    Municipality,
    MunicipalityWithDistance,
    OfflineMunicipality,
    OfflineMunicipalityWithDistance,
)


def search_close_meeting_points(
    all_points: List[Municipality],
    latitude: float,
    longitude: float,
    radius_km: int,
) -> List[MunicipalityWithDistance]:
    """
    Search the closest meeting points.
    Args:
        all_points (List[Municipality]): list of the municipality.
        latitude (float): latitude of the search.
        longitude (float): longitude of the search.
        radius_km (int): radius_km of the search.
    Returns:
        List[MunicipalityWithDistance]: The list of closest meeting point.
    """
    _logger = logging.getLogger("root")

    close_points: List[MunicipalityWithDistance] = []

    for point in all_points:
        try:
            copy_point = point.copy()
            copy_point["distance_km"] = round(
                distance.distance(
                    (latitude, longitude), (point["latitude"], point["longitude"])
                ).km,
                2,
            )
            if copy_point["distance_km"] < radius_km:
                close_points.append(copy_point)
        except Exception as geolocalisation_e:
            _logger.error(
                "Error while calculating distance from %s : %s",
                point["name"],
                geolocalisation_e,
            )
    return close_points


def search_close_offline_meeting_points(
    all_points: List[OfflineMunicipality],
    latitude: float,
    longitude: float,
    radius_km: int,
) -> List[OfflineMunicipalityWithDistance]:
    """
    Search the closest offline meeting points.
    Args:
        all_points (List[OfflineMunicipality]): list of the offline municipalities.
        latitude (float): latitude of the search.
        longitude (float): longitude of the search.
        radius_km (int): radius_km of the search.
    Raises:
        HTTPException: Raise http-400 when there is bad type of params.
    Returns:
        List[OfflineMunicipalityWithDistance]: The list of closest offline meeting points.
    """
    close_points: List[OfflineMunicipalityWithDistance] = []
    try:
        for point in all_points:
            copy_point = point.copy()
            copy_point["distance"] = round(
                distance.distance(
                    (latitude, longitude), (point["latitude"], point["longitude"])
                ).km,
                2,
            )
            if copy_point["distance"] < radius_km:
                close_points.append(copy_point)
    except Exception:
        raise HTTPException(
            status_code=HTTP_400_BAD_REQUEST, detail="Bad type of params"
        )
    return close_points



import asyncio
import os
from datetime import date
from typing import Any, List

from fastapi import APIRouter, Depends, HTTPException, Query, Request, Response
from markdown import markdown
from pydantic import Required
from slowapi import Limiter
from slowapi.util import get_remote_address
from sqlalchemy.orm import sessionmaker

from ...db.postgresdb_utils import get_database
from ...db.utils import (
    get_all_editors,
    get_all_meeting_points,
    get_all_offline_meeting_points,
)
from ...models.announcement import Announcement
from ...models.municipality import (
    Municipality,
    MunicipalityWithDistance,
    OfflineMunicipality,
    OfflineMunicipalityWithDistance,
)
from ...services.panorama_service import get_all_editor_panorama
from ...services.search_meeting_points import (
    search_close_meeting_points,
    search_close_offline_meeting_points,
)
from ..dependencies.auth_token import verify_internal_auth_token

router = APIRouter()

limiter = Limiter(key_func=get_remote_address)


@router.get(
    "/MeetingPointsFromPosition",
    response_model=List[MunicipalityWithDistance],
    responses={
        200: {
            "description": "Les Meetings points ont bien été trouvés",
            "content": {
                "application/json": {
                    "example": [
                        {
                            "id": "201",
                            "name": "Mairie ANNEXE LILLE-SECLIN",
                            "longitude": 3.0348016639327,
                            "latitude": 50.549140395451,
                            "public_entry_address": "89 RUE ROGER BOUVRY",
                            "zip_code": "59113",
                            "city_name": "SECLIN",
                            "website": "https://www.ville-seclin.fr",
                            "city_logo": "https://www.ville-seclin.fr/images/logo-ville-seclin/logo_ville_de_seclin.png",
                            "distance_km": 1.56,
                        }
                    ]
                }
            },
        }
    },
)
@limiter.limit("30/minute")
def meeting_points_from_position(
    request: Request,
    longitude: float = Query(default=Required, example=2.352222),
    latitude: float = Query(default=Required, example=48.856613),
    radius_km: int = Query(default=20, enum=[20, 40, 60]),
) -> List[MunicipalityWithDistance]:
    """
    Recherche des meeting points à partir d'une coordonnée GPS.
    """
    all_points: List[Municipality] = get_all_meeting_points()
    meeting_points: List[MunicipalityWithDistance] = search_close_meeting_points(
        all_points, latitude, longitude, radius_km
    )
    return meeting_points


@router.get(
    "/Announcement",
    response_model=Announcement,
    responses={
        200: {
            "description": "System global announcement",
            "content": {
                "application/json": {
                    "example": [
                        {
                            "title": "Alerte de sécurité",
                            "description": "Attention, le site rencontre des attaques recurrentes en ce moment.",
                            "alert_level": "error",
                        }
                    ]
                }
            },
        }
    },
)
@limiter.limit("30/minute")
def global_announcement(request: Request) -> Any:
    """
    Retourne un objet d'annonce globale.
    """
    if (
        ("ANNOUNCEMENT_TITLE" in os.environ)
        and ("ANNOUNCEMENT_DESCRIPTION" in os.environ)
        and ("ANNOUNCEMENT_ALERT_LEVEL" in os.environ)
    ):
        return {
            "title": os.environ.get("ANNOUNCEMENT_TITLE"),
            "description": markdown(os.environ.get("ANNOUNCEMENT_DESCRIPTION")),
            "alert_level": os.environ.get("ANNOUNCEMENT_ALERT_LEVEL"),
        }
    return Response(status_code=200)


@router.get(
    "/searchCity",
    response_model=Municipality,
    responses={
        200: {
            "description": "La ville a bien été trouvée",
            "content": {
                "application/json": {
                    "example": {
                        "id": "201",
                        "name": "Annexe mairie des Favignolles à Romorantin",
                        "longitude": 1.751597,
                        "latitude": 47.349797,
                        "public_entry_address": "1 rue François RABELAIS",
                        "zip_code": "41200",
                        "city_name": "Romorantin-Lanthenay",
                        "decoded_city_name": "romorantin-lanthenay",
                        "website": "https://rendezvousonline.fr/alias/romorantin-lanthenay-41200-2",
                        "city_logo": "https://pro.rendezvousonline.fr/upload/account/image/logo/2G8VjHfrJWB93mYFhPiYO5F4bPRJdaJz.jpg",
                    }
                }
            },
        }
    },
)
@limiter.limit("30/minute")
def search_city(
    request: Request,
    name: str = Query(default=Required, example="romorantin-lanthenay"),
) -> Municipality:
    """
    Recherche les meeting points d'une ville.
    """
    all_points: List[Municipality] = get_all_meeting_points()
    city = None
    if name == "lyon":
        for point in all_points:
            if point["name"] == "Mairie du 3ème arrondissement":
                city = point
                return city
    elif name == "paris":
        for point in all_points:
            if point["name"] == "Service titres d'identité de Paris  Centre":
                city = point
                return city
    elif name == "marseille":
        for point in all_points:
            if point["name"] == "Mairie de Marseille : BMdP Désirée Clary":
                city = point
                return city
    else:
        for point in all_points:
            if point["decoded_city_name"] == name:
                city = point
                return city
    return city


@router.get(
    "/status",
    responses={
        200: {
            "description": "Informations des éditeurs",
            "content": {
                "application/json": {
                    "example": {
                        "editors": [
                            {
                                "slug": "troov",
                                "name": "Troov",
                                "api_url": "https://qa-api.troov.com/api",
                                "status": True,
                            }
                        ]
                    }
                }
            },
        }
    },
)
@limiter.limit("30/minute")
async def get_editors(request: Request) -> Any:
    """
    Donne les détails des éditeurs.
    """
    all_editors = get_all_editors()
    editors = []
    for editor in all_editors:
        editors.append(editor.convert_to_front())
    return {"editors": editors}


@router.get(
    "/searchOfflineMeetingPoints",
    response_model=List[OfflineMunicipalityWithDistance],
    responses={
        200: {
            "description": "La recherche des meeting points hors ligne a bien aboutie",
            "content": {
                "application/json": {
                    "example": [
                        {
                            "id": "201",
                            "nmunicipality": "Paris 10",
                            "public_entry_address": "72 Rue du Faubourg Saint-Martin",
                            "phone_number": "04 14 04 44 44",
                            "decoded_city_name": "paris",
                            "url": "https://urldumeetingpoint.fr",
                            "logo": "https://urldulogo.fr",
                            "website": "https://mairie10.paris.fr",
                            "distance_km": 1.56,
                        }
                    ]
                }
            },
        }
    },
)
@limiter.limit("30/minute")
def search_offline_meeting_points(
    request: Request,
    longitude: float = Query(default=Required, example=2.352222),
    latitude: float = Query(default=Required, example=48.856613),
    radius_km: float = Query(default=20, enum=[0.99, 5, 10, 15, 20, 25, 30]),
) -> List[OfflineMunicipalityWithDistance]:
    """
    Recherche des meeting points hors ligne autour d'une position GPS
    """
    all_offline_points: List[OfflineMunicipality] = get_all_offline_meeting_points()
    meeting_points: List[
        OfflineMunicipalityWithDistance
    ] = search_close_offline_meeting_points(
        all_offline_points, latitude, longitude, radius_km
    )
    return meeting_points


@router.get(
    "/internalAvailableTimeSlots",
    include_in_schema=False,
    responses={
        200: {
            "description": "Les rendez-vous disponibles pour les meeting points id ont bien été trouvés",
            "content": {
                "application/json": {
                    "example": {
                        "506": [
                            {
                                "datetime": "2022-12-19T10:00Z",
                                "callback_url": "https://www.ville-seclin.fr/rendez-vous/passeports?date=2022-12-19T10:00Z",
                            },
                            {
                                "datetime": "2022-12-19T10:20Z",
                                "callback_url": "https://www.ville-seclin.fr/rendez-vous/passeports?date=2022-12-19T10:20Z",
                            },
                            {
                                "datetime": "2022-12-19T10:40Z",
                                "callback_url": "https://www.ville-seclin.fr/rendez-vous/passeports?date=2022-12-19T10:40Z",
                            },
                            {
                                "datetime": "2022-12-19T11:00Z",
                                "callback_url": "https://www.ville-seclin.fr/rendez-vous/passeports?date=2022-12-19T11:00Z",
                            },
                        ],
                        "7789": [
                            {
                                "datetime": "2022-12-19T10:00Z",
                                "callback_url": "https://www.lille.fr/Lille-Sud2/Mairie-de-quartier-de-Lille-Sud/rendez-vous/passeports?date=2022-12-19T10:00Z",
                            },
                            {
                                "datetime": "2022-12-19T10:20Z",
                                "callback_url": "https://www.lille.fr/Lille-Sud2/Mairie-de-quartier-de-Lille-Sud/rendez-vous/passeports?date=2022-12-19T10:20Z",
                            },
                            {
                                "datetime": "2022-12-19T10:40Z",
                                "callback_url": "https://www.lille.fr/Lille-Sud2/Mairie-de-quartier-de-Lille-Sud/rendez-vous/passeports?date=2022-12-19T10:40Z",
                            },
                            {
                                "datetime": "2022-12-19T11:00Z",
                                "callback_url": "https://www.lille.fr/Lille-Sud2/Mairie-de-quartier-de-Lille-Sud/rendez-vous/passeports?date=2022-12-19T11:00Z",
                            },
                        ],
                    }
                }
            },
        },
        "401": {
            "description": "Utilisateur non autorisé",
            "content": {
                "application/json": {"example": "X-HUB-RDV-AUTH-TOKEN header invalid"}
            },
        },
        "404": {
            "description": "Meeting point id non trouvé",
            "content": {
                "application/json": {"example": "Unknown Meeting Point ID : 11TEST2111"}
            },
        },
    },
    dependencies=[Depends(verify_internal_auth_token)],
)
async def get_available_time_slots(
    request: Request,
    meeting_point_ids: list[str] = Query(default=Required, example=["201", "203"]),
    start_date: date = Query(default=Required, example="2022-11-01"),
    end_date: date = Query(default=Required, example="2022-11-30"),
    reason: str = Query(default="CNI", enum=["CNI", "PASSPORT", "CNI-PASSPORT"]),
    documents_number: int = Query(default=1),
):
    """
    Récupère les rendez-vous disponibles sur une période donnée pour une liste de meeting point id.
    <br>
    Attention: L'heure qui sera affichée est l'heure exacte que l'on trouve dans la réponse.
    """

    all_points = get_all_meeting_points()

    result = {}

    meeting_points = []
    for meeting_point_id in meeting_point_ids:
        meeting_point = None
        index = 0
        while index < len(all_points):
            if meeting_point_id == all_points[index]["id"]:
                meeting_point = all_points[index]
                break
            index += 1

        if not meeting_point:
            raise HTTPException(
                status_code=404, detail=f"Unknown Meeting Point ID : {meeting_point_id}"
            )
        meeting_points.append(meeting_point)
    editor_futures = []
    for editor in get_all_editors():
        editor_futures.append(
            asyncio.ensure_future(
                editor.search_slots_in_editor(
                    meeting_points,
                    start_date,
                    end_date,
                    reason,
                    documents_number,
                    None,
                )
            )
        )
    all_editor_time_slots = await asyncio.gather(*editor_futures)
    for editor_time_slots in all_editor_time_slots:
        for meeting_point_slots in editor_time_slots[0]:
            result[meeting_point_slots["id"]] = meeting_point_slots["available_slots"]

    return result


@router.get(
    "/panorama",
    responses={
        200: {
            "description": "Panorama des éditeurs",
            "content": {
                "application/json": {
                    "example": [
                        {
                            "id": 1,
                            "site_url": "https://www.urlDeLediteurconcerne.fr/",
                            "overseas": False,
                            "limit_appointment_outside": True,
                            "use_queue": False,
                            "is_connect_to_anti_duplication": True,
                            "contact_url": "https://www.urlDeLediteurconcerne.fr/contact",
                            "name": "Nom de l'editeur",
                            "logo_url": "https://www.urlDeLediteurconcerne.fr/logo.png",
                            "integration_predemande": True,
                            "saas": True,
                            "is_connect_to_hub": True,
                        },
                    ]
                }
            },
        }
    },
)
@limiter.limit("30/minute")
async def get_editors_panorama(
    request: Request, session: sessionmaker = Depends(get_database)
) -> Any:
    """
    Récupère les données des éditeurs de la page panorama

    Retourne:
        La liste de tous les éditeurs.
    """
    try:
        all_editors = get_all_editor_panorama(session)
        return all_editors
    except Exception as e:
        raise HTTPException(
            status_code=500,
            detail=f"Error while getting editors panorama : {str(e)}",
        )
    finally:
        session.close()



# pylint: disable=C0415
import asyncio
import json
import logging
import random
import threading
import time
from datetime import date, datetime, timedelta

import httpx
from dateutil import tz
from fastapi import WebSocket
from httpx import ConnectTimeout, ReadTimeout
from pydantic import ValidationError

from ..core.custom_validation_error import CustomValidationError
from ..logging.app_logger import write_external_service_data
from ..models.token_ants_to_editor import TokenAntsToEditor
from ..models.token_editor_to_ants import TokenEditorToAnts
from ..services.data_validator import capitalize_custom, is_date_after_paris_datetime
from ..services.mock_data import get_mock_managed_meeting_points, get_mock_slots
from .application import Application
from .municipality import Municipality

FRA_TZ = tz.gettz("Europe/Paris")


class Editor:
    slug: str
    name: str
    url: str
    _test_mode: bool
    header_name: str
    tokens_editor_to_ants: list = [TokenEditorToAnts]
    tokens_ants_to_editor: list = [TokenAntsToEditor]
    status: bool
    api_down_datetime: datetime
    api_up_datetime: datetime
    only_connect_to_anti_duplication: bool

    def __init__(
        self,
        name: str,
        url: str,
        test_mode: bool,
        header_name: str,
        tokens_editor_to_ants: list = [TokenEditorToAnts],
        tokens_ants_to_editor: list = [TokenAntsToEditor],
        status: bool = True,
        api_down_datetime: datetime = None,
        api_up_datetime: datetime = None,
        only_connect_to_anti_duplication: bool = False,
    ):
        self.slug = name.lower().replace(" ", "-")
        self.name = name
        self.url = url
        self._test_mode = test_mode
        self.status = status
        self.api_down_datetime = api_down_datetime
        self.api_up_datetime = api_up_datetime
        self.header_name = header_name
        self.tokens_editor_to_ants = tokens_editor_to_ants
        self.tokens_ants_to_editor = tokens_ants_to_editor
        self.only_connect_to_anti_duplication = only_connect_to_anti_duplication

    async def get_managed_meeting_points(self):
        _logger = logging.getLogger("root")
        await asyncio.sleep(0.00001)
        points = []
        if self._test_mode:
            points = get_mock_managed_meeting_points(self)
        else:
            try:
                headers = {}
                headers[self.header_name] = self.tokens_ants_to_editor[0].token
                async with httpx.AsyncClient(verify=False) as async_client:
                    response = await async_client.get(
                        f"{self.url}/getManagedMeetingPoints",
                        headers=headers,
                        timeout=20,
                        follow_redirects=True,
                    )
                    write_external_service_data(
                        _logger, response, editor_name=self.name
                    )
                    if response.status_code in [200]:
                        self.api_up_datetime = datetime.now(tz=FRA_TZ)
                        if not self.status:
                            self.status = True

                        points = response.json()
                    else:
                        raise Exception(
                            f"Status code not handled : {response.status_code}"
                        )
            except Exception as get_meeting_points_e:
                if self.status:
                    self.status = False
                    self.api_down_datetime = datetime.now(tz=FRA_TZ)
                _logger.error(
                    "Error while getting meeting points",
                    extra={
                        "extra_info": {
                            "type": "app",
                            "editor_name": self.name,
                            "category": "external_service",
                            "endpoint": "getManagedMeetingPoints",
                            "error": str(get_meeting_points_e),
                        }
                    },
                )

                retry_thread = [
                    th
                    for th in threading.enumerate()
                    if th.name == f"retry-get-managed-{self.name}"
                ]
                if len(retry_thread) < 1:
                    retry_thread = threading.Thread(
                        target=self.retry_get_managed_meeting_points,
                        name=f"retry-get-managed-{self.name}",
                    )
                    retry_thread.start()

        valid_meeting_points = []
        for point in points:
            try:
                point["_editor_name"] = self.name
                point["_internal_id"] = str(point["id"])
                point["city_name"] = capitalize_custom(point["city_name"])
                if (
                    point["zip_code"]
                    and point["public_entry_address"]
                    and (point["zip_code"] in point["public_entry_address"])
                ):
                    point["public_entry_address"] = point["public_entry_address"][
                        : point["public_entry_address"].index(point["zip_code"])
                    ].strip()
                point["zip_code"] = point["zip_code"] and point["zip_code"].strip()
                Municipality.parse_obj(point)
                valid_meeting_points.append(point)
            except ValidationError as meeting_point_validation_e:
                _logger.error(
                    "Error while validating meeting point : %s \nError: %s",
                    point,
                    meeting_point_validation_e,
                    extra={"extra_info": {"type": "app", "editor_name": self.name}},
                )
            except Exception as validation_unknown_e:
                _logger.error(
                    "Error while validating meeting point : %s \nError: %s",
                    point,
                    validation_unknown_e,
                    extra={"extra_info": {"type": "app", "editor_name": self.name}},
                )

        return valid_meeting_points

    async def get_available_time_slots(
        self, meeting_points, start_date, end_date, reason="CNI", documents_number=1
    ):
        _logger = logging.getLogger("root")
        # this sleep is necessary to not block other async operations
        await asyncio.sleep(0.00001)
        result = {}
        editor_error = None
        response = None

        def last_day_of_month(any_day):
            next_month = any_day.replace(day=28) + timedelta(days=4)
            return (next_month.replace(day=1) - timedelta(days=1)).day

        # start_date and end_date should be fixed to help editors handle their cache
        monthly_start_date = None
        monthly_end_date = None
        try:
            monthly_start_date = start_date.replace(
                day=date.today().day if start_date.month == date.today().month else 1
            )
            monthly_end_date = end_date.replace(day=last_day_of_month(end_date))
        except Exception as monthly_date_e:
            _logger.error(
                "Error while creating monthly search dates: %s",
                monthly_date_e,
                extra={"extra_info": {"type": "app"}},
            )

        if self._test_mode:
            await asyncio.sleep(random.randint(3, 12))
            for meeting_point in meeting_points:
                meeting_point_slots = get_mock_slots(
                    meeting_point, start_date, end_date
                )
                result[meeting_point["_internal_id"]] = meeting_point_slots
        else:
            meeting_point_ids = [x["_internal_id"] for x in meeting_points]
            try:
                headers = {}
                headers[self.header_name] = self.tokens_ants_to_editor[0].token
                parameters = {
                    "start_date": start_date or monthly_start_date or date.today(),
                    "end_date": end_date
                    or monthly_end_date
                    or (date.today() + timedelta(days=150)),
                    "meeting_point_ids": meeting_point_ids,
                    "reason": reason,
                    "documents_number": documents_number,
                }
                async with httpx.AsyncClient(verify=False) as async_client:
                    response = await async_client.get(
                        f"{self.url}/availableTimeSlots",
                        headers=headers,
                        params=parameters,
                        timeout=15,
                        follow_redirects=True,
                    )
                    write_external_service_data(_logger, response, self.name)
                    if response.status_code in [200]:
                        self.api_up_datetime = datetime.now(tz=FRA_TZ)
                        if not self.status:
                            self.status = True
                            self.api_down_datetime = datetime.now(tz=FRA_TZ)

                        result = response.json()
                    else:
                        raise Exception(
                            f"Status code not handled : {response.status_code} : {response.reason_phrase}"
                        )
            except ReadTimeout:
                if self.status:
                    self.status = False
                    self.api_down_datetime = datetime.now(tz=FRA_TZ)

                _logger.error(
                    "Timeout while getting available time slots for %s",
                    self.name,
                    extra={
                        "extra_info": {
                            "type": "app",
                            "editor_name": self.name,
                            "category": "external_service",
                            "endpoint": "availableTimeSlots",
                        }
                    },
                )
            except ConnectTimeout:
                if self.status:
                    self.status = False
                    self.api_down_datetime = datetime.now(tz=FRA_TZ)

                _logger.error(
                    "Connexion Timeout while getting available time slots for %s",
                    self.name,
                    extra={
                        "extra_info": {
                            "type": "app",
                            "editor_name": self.name,
                            "category": "external_service",
                            "endpoint": "availableTimeSlots",
                        }
                    },
                )
            except Exception as available_time_slots_e:
                if self.status:
                    self.status = False
                    self.api_down_datetime = datetime.now(tz=FRA_TZ)

                _logger.error(
                    "Error while getting available time slots for %s : %s",
                    self.name,
                    str(available_time_slots_e),
                    extra={
                        "extra_info": {
                            "type": "app",
                            "parameters": {
                                "start_date": str(parameters["start_date"]),
                                "end_date": str(parameters["end_date"]),
                                "meeting_point_ids": parameters["meeting_point_ids"],
                                "reason": parameters["reason"],
                                "documents_number": parameters["documents_number"],
                            },
                            "editor_name": self.name,
                            "category": "external_service",
                            "endpoint": "availableTimeSlots",
                        }
                    },
                )
                editor_error = {"error": available_time_slots_e, "editor": self.name}

        if (not start_date) and (not end_date):
            return result, None

        filtered_dates_result = {}
        try:
            for meeting_point_id in result:
                filtered_dates_result[meeting_point_id] = []
                for available_timeslot in result[meeting_point_id]:
                    timeslot_datetime = None
                    for datetime_format in ["%Y-%m-%dT%H:%MZ", "%Y-%m-%dT%H:%M:%S%z"]:
                        try:
                            timeslot_datetime = datetime.strptime(
                                available_timeslot["datetime"], datetime_format
                            )

                            if timeslot_datetime.tzinfo:
                                utcoffset = (
                                    timeslot_datetime.utcoffset().total_seconds()
                                    / 60
                                    / 60
                                )
                                timeslot_datetime = timeslot_datetime.replace(
                                    tzinfo=None
                                ) + timedelta(hours=utcoffset)

                            break
                        except ValueError:
                            pass
                    if (
                        (not timeslot_datetime)
                        or (start_date and (timeslot_datetime.date() < start_date))
                        or (end_date and (timeslot_datetime.date() > end_date))
                        or (is_date_after_paris_datetime(timeslot_datetime))
                    ):
                        message = "DATE OUT OF RANGE"
                        if is_date_after_paris_datetime(timeslot_datetime):
                            message = "DATE IN THE PAST"
                        _logger.debug(
                            "[%s] %s : %s",
                            self.name,
                            message,
                            str(timeslot_datetime),
                        )
                    else:
                        filtered_dates_result[meeting_point_id].append(
                            {
                                "datetime": timeslot_datetime.strftime(
                                    "%Y-%m-%dT%H:%M:%SZ"
                                ),
                                "callback_url": available_timeslot["callback_url"],
                            }
                        )
        except Exception as checking_date_filter_e:
            _logger.error(
                "[%s] Checking date filter error : %s",
                self.name,
                str(checking_date_filter_e),
                extra={"extra_info": {"type": "app", "editor_name": self.name}},
            )
            editor_error = {"error": checking_date_filter_e, "editor": self.name}
        return filtered_dates_result, editor_error

    async def search_slots_in_editor(
        self,
        meeting_points,
        start_date,
        end_date,
        reason="CNI",
        documents_number=1,
        websocket: WebSocket = None,
    ):
        # this sleep is necessary to not block other async operations
        time.sleep(0.00001)
        editor_error = None
        editor_meeting_points = []
        editor_meeting_points_with_slots = []
        for meeting_point in meeting_points:
            if meeting_point["_editor_name"] == self.name:
                editor_meeting_points.append(meeting_point)
        if editor_meeting_points:
            slots, editor_error = await self.get_available_time_slots(
                editor_meeting_points, start_date, end_date, reason, documents_number
            )
            for meeting_point in editor_meeting_points:
                if (
                    meeting_point["_internal_id"] in slots
                    and slots[meeting_point["_internal_id"]]
                ):
                    meeting_point["available_slots"] = slots[
                        meeting_point["_internal_id"]
                    ]
                    editor_meeting_points_with_slots.append(meeting_point)
            if websocket:
                safe_editor_meeting_points_with_slots = []
                for editor_meeting_point_with_slots in editor_meeting_points_with_slots:
                    editor_meeting_point_with_slots_copy = (
                        editor_meeting_point_with_slots.copy()
                    )
                    if "_editor_name" in editor_meeting_point_with_slots_copy:
                        del editor_meeting_point_with_slots_copy["_editor_name"]
                    if "_internal_id" in editor_meeting_point_with_slots_copy:
                        del editor_meeting_point_with_slots_copy["_internal_id"]
                    safe_editor_meeting_points_with_slots.append(
                        editor_meeting_point_with_slots_copy
                    )
                json_string = json.dumps(
                    safe_editor_meeting_points_with_slots, default=str
                )
                await websocket.send_text(json_string)
        return editor_meeting_points_with_slots, editor_error

    async def search_meetings(self, application_ids):
        _logger = logging.getLogger("root")
        await asyncio.sleep(0.00001)
        meetings = {}
        if not self._test_mode:
            try:
                headers = {}
                headers[self.header_name] = self.tokens_ants_to_editor[0].token
                parameters = {"application_ids": application_ids}
                async with httpx.AsyncClient(verify=False) as async_client:
                    response = await async_client.get(
                        f"{self.url}/searchApplicationIds",
                        headers=headers,
                        params=parameters,
                        timeout=5,
                        follow_redirects=True,
                    )
                    write_external_service_data(_logger, response, self.name)
                    if response.status_code in [200]:
                        meetings = response.json()
                    else:
                        raise Exception(
                            f"Status code not handled : {response.status_code} : {response.reason_phrase}"
                        )
            except Exception as search_meetings_e:
                _logger.error(
                    "Error while searching meetings by application ID for %s : %s",
                    self.name,
                    str(search_meetings_e),
                    extra={
                        "extra_info": {
                            "type": "app",
                            "editor_name": self.name,
                            "endpoint": "searchApplicationIds",
                        }
                    },
                )
        else:
            await asyncio.sleep(random.randint(3, 5))

        valid_meetings = {}
        for applicationId in meetings:
            valid_meetings[applicationId] = []
            for meeting in meetings[applicationId]:
                try:
                    Application.parse_obj(meeting)
                    valid_meetings[applicationId].append(meeting)
                except ValidationError as meeting_validation_e:
                    _logger.info(
                        "Error while validating meeting : %s \nError: %s",
                        meeting,
                        meeting_validation_e,
                        extra={
                            "extra_info": {
                                "type": "app",
                                "editor_name": self.name,
                                "endpoint": "searchApplicationIds",
                            }
                        },
                    )
        return valid_meetings

    def retry_get_managed_meeting_points(self):
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)

        result = loop.run_until_complete(self._retry_get_managed_meeting_points())
        loop.close()

        return result

    async def _retry_get_managed_meeting_points(self):
        _logger = logging.getLogger("root")

        counter = 0
        while counter < 60 * 20:
            time.sleep(1 * 60)
            try:
                meeting_points = await self.get_managed_meeting_points()
                if meeting_points:
                    from unidecode import unidecode

                    from ..db.utils import (
                        get_all_meeting_points,
                        set_all_meeting_points,
                    )

                    all_meeting_points = get_all_meeting_points()

                    point_index = len(all_meeting_points) + 1
                    for point in meeting_points:
                        point["id"] = str(point_index)
                        point["decoded_city_name"] = (
                            unidecode(point["city_name"])
                            .replace(" ", "-")
                            .replace("'", "-")
                            .lower()
                        )
                        point_index += 1

                    all_meeting_points += meeting_points
                    set_all_meeting_points(all_meeting_points)
                    break
                else:
                    raise Exception(f"No meeting points found for editor {self.name}")
            except CustomValidationError as retry_get_managed_meeting_points_e:
                _logger.error(
                    "Error while retrying to get managed meeting for editor %s : %s",
                    self.name,
                    retry_get_managed_meeting_points_e,
                    extra={
                        "extra_info": {
                            "type": "app",
                            "editor_name": self.name,
                            "category": "retrying",
                        }
                    },
                )
            counter += 1

        _logger.info(
            "End of retrying to get managed meeting points for editor %s",
            self.name,
            extra={"extra_info": {"type": "app"}},
        )

    def convert_to_front(self):
        return {
            "slug": self.slug,
            "name": self.name,
            "status": self.status,
            "api_down_datetime": self.api_down_datetime,
            "api_up_datetime": self.api_up_datetime,
        }


def init_all_editors():
    from ..crud.crud_editor import editor as crud
    from ..db.postgresdb import session

    try:
        editors = crud.get_editors_not_only_opti(db=session)
        editors_list = []
        if editors:
            for editor in editors:
                dto = editor.to_dto()
                editors_list.append(dto)
        else:
            print("No editors found in database")
        print("------- Done loading editors -------")
        return editors_list
    except Exception as e:
        print("Error while getting editors : ", str(e))
    finally:
        session.close()



import datetime

from pydantic import BaseModel


class TokenEditorToAntsBase(BaseModel):
    """
    Base class for token editor to ants schema.
    """

    id_editor: int
    token: str
    validity_date: datetime.datetime


class TokenEditorToAntsCreate(TokenEditorToAntsBase):
    """
    Class for creating token editor to ants schema.
    """


class TokenEditorToAntsUpdate(TokenEditorToAntsBase):
    """
    Class for updating token editor to ants schema.
    """


class TokenEditorToAnts(TokenEditorToAntsBase):
    """
    Class for token editor to ants schema.
    """

    id: int

    class Config:
        orm_mode = True



from pydantic import BaseModel


class EditorPanoramaBase(BaseModel):
    """
    Base editor schema.
    """

    name: str
    logo_url: str
    site_url: str
    contact_url: str
    overseas: bool
    integration_predemande: bool
    limit_appointment_outside: bool
    saas: bool
    use_queue: bool
    is_connect_to_hub: bool
    is_connect_to_anti_duplication: bool


class EditorPanoramaCreate(EditorPanoramaBase):
    """
    Editor create schema.
    """


class EditorPanoramaUpdate(EditorPanoramaBase):
    """
    Editor update schema.
    """


class EditorPanorama(EditorPanoramaBase):
    """
    Editor schema.
    """

    id: int

    class Config:
        orm_mode = True



import logging
from typing import List

from src.hubrdvmairie.models.editor_panorama import EditorPanorama

from ..crud.crud_editor_panorama import editorPanorama as crud

logger = logging.getLogger(__name__)


def get_all_editor_panorama(session) -> List[EditorPanorama]:
    try:
        return crud.get_all(session)
    except Exception as e:
        logger.error("Error while getting all editor panorama : %s", str(e))



from sqlalchemy import Boolean, Column, Integer, String
from sqlalchemy.orm import relationship

from ..db.postgresdb import Base
from .meeting_point import MeetingPoint  # noqa: F401
from .token_ants_to_editor import TokenAntsToEditor  # noqa: F401
from .token_editor_to_ants import TokenEditorToAnts  # noqa: F401


class Editor(Base):
    """
    Represents an editor entity in the database.
    """

    __tablename__ = "editor"

    id = Column(Integer, primary_key=True, index=True)
    name = Column(String, unique=True, nullable=False)
    url = Column(String, nullable=False)
    header_name = Column(String, nullable=False)
    only_connect_to_anti_duplication = Column(Boolean)
    tokens_ants_to_editor = relationship("TokenAntsToEditor", back_populates="editor")
    tokens_editor_to_ants = relationship("TokenEditorToAnts", back_populates="editor")
    meeting_points = relationship("MeetingPoint", back_populates="editor")

    def clean_token(self):
        """
        Removes invalid tokens from the lists of tokens.
        """

        self.tokens_ants_to_editor = [
            token for token in self.tokens_ants_to_editor if token.is_valid()
        ]
        self.tokens_editor_to_ants = [
            token for token in self.tokens_editor_to_ants if token.is_valid()
        ]

    def to_dto(self):
        """
        Converts the Editor object to a DTO object.
        Clear token list before conversion: a token is valid if the validity date is in the future or not set.
        Returns:
            DTO: A DTO object representing the Editor.
        """

        from .editor import Editor as DTO  # noqa: F401

        self.clean_token()
        return DTO(
            url=self.url,
            name=self.name,
            test_mode=False,
            header_name=self.header_name,
            tokens_editor_to_ants=self.tokens_editor_to_ants,
            tokens_ants_to_editor=self.tokens_ants_to_editor,
            status=False,
        )



from typing import List, Optional

from sqlalchemy.orm import Session

from ..crud.base import CRUDBase
from ..models.editor_panorama import EditorPanorama
from ..schemas.editor_panorama import EditorPanoramaCreate, EditorPanoramaUpdate


class CRUDEditorPanorama(
    CRUDBase[EditorPanorama, EditorPanoramaCreate, EditorPanoramaUpdate]
):
    """
    CRUD operations for the Editor model.
    """

    def get_by_name(self, db: Session, *, name: str) -> Optional[EditorPanorama]:
        """
        Retrieve an editor by name.

        Args:
            db (Session): The database session.
            name (str): The name of the editor to retrieve.

        Returns:
            Optional[Editor]: The editor with the given name, or None if not found.
        """
        return db.query(EditorPanorama).filter(EditorPanorama.name == name).first()

    def get_all(self, db: Session) -> List[EditorPanorama]:
        """
        Retrieve all editors.

        Args:
            db (Session): The database session.

        Returns:
            List[Editor]: A list of all editors.
        """
        return db.query(EditorPanorama).all()


editorPanorama = CRUDEditorPanorama(EditorPanorama)



from typing import List, Optional

from sqlalchemy.orm import Session

from ..crud.base import CRUDBase
from ..models.token_editor_to_ants import TokenEditorToAnts
from ..schemas.token_editor_to_ants import (
    TokenEditorToAntsCreate,
    TokenEditorToAntsUpdate,
)


class CRUDTokenEditorToAnts(
    CRUDBase[TokenEditorToAnts, TokenEditorToAntsCreate, TokenEditorToAntsUpdate]
):
    """
    CRUD operations for TokenEditorToAnts model.
    """

    def get_by_token(self, db: Session, *, token: str) -> Optional[TokenEditorToAnts]:
        """
        Retrieve a token by its value.

        Args:
            db (Session): SQLAlchemy session object.
            token (str): Token value.

        Returns:
            Optional[TokenEditorToAnts]: TokenEditorToAnts object if found, else None.
        """
        return (
            db.query(TokenEditorToAnts).filter(TokenEditorToAnts.token == token).first()
        )

    def get_all(self, db: Session):
        """
        Retrieve all tokens.

        Args:
            db (Session): SQLAlchemy session object.

        Returns:
            List[TokenEditorToAnts]: List of all TokenEditorToAnts objects.
        """
        return db.query(TokenEditorToAnts).all()

    def get_valid_token(self, db: Session) -> List[TokenEditorToAnts]:
        """
        Retrieve all tokens.

        Args:
            db (Session): SQLAlchemy session object.

        Returns:
            List[TokenEditorToAnts]: List of all TokenEditorToAnts objects.
        """
        tokens: List[TokenEditorToAnts] = db.query(TokenEditorToAnts).all()
        valid_token = [token for token in tokens if token.is_valid()]
        return valid_token


tokenEditorToAnts = CRUDTokenEditorToAnts(TokenEditorToAnts)



import logging
import sys
from http import HTTPStatus
from urllib.parse import quote

from fastapi import Request, Response
from httpx import Response as HTTPResponse


def get_stream_handler(formatter):
    stream_handler = logging.StreamHandler(sys.stdout)
    stream_handler.setLevel(logging.DEBUG)
    stream_handler.setFormatter(formatter)
    return stream_handler


def get_logger(name, formatter):
    logger = logging.getLogger(name)
    if not logger.handlers:
        logger.setLevel(logging.DEBUG)
        logger.addHandler(get_stream_handler(formatter))
    return logger


def get_extra_info(request: Request, response: Response):
    status_reasons = {x.value: x.name for x in list(HTTPStatus)}
    return {
        "req": {
            "url": quote(request.url.path, safe="/:"),
            "headers": {
                "host": quote(request.headers.get("host", ""), safe=":"),
                "accept": quote(request.headers.get("accept", ""), safe="/ ,*+=;")
                or None,
            },
            "method": request.method,
            "httpVersion": request.scope["http_version"],
            "originalUrl": quote(request.url.path, safe="/:"),
            "query": {},
        },
        "res": {
            "statusCode": response.status_code,
            "body": {
                "statusCode": response.status_code,
                "status": status_reasons.get(response.status_code),
            },
        },
        "protocol": "http",
        "response_time": response.response_time,
        "realip": request.client.host,
        "source_side": "Backend",
        "type": "access",
    }


def get_external_service_extra_info(response: HTTPResponse, editor_name=None):
    url_path = (
        response.request.url.scheme
        + "://"
        + response.request.url.host
        + response.request.url.path
    )
    return {
        "category": "external_service",
        "request": {"url": url_path, "method": response.request.method},
        "response": {"statusCode": response.status_code},
        "protocol": "http",
        "response_time": response.elapsed.microseconds / 1000,
        "source_side": "Backend",
        "type": "app",
        "editor_name": editor_name,
    }


def write_access_log_data(logger, request, response):
    try:
        logger.info(
            request.method + " " + request.url.path,
            extra={"extra_info": get_extra_info(request, response)},
        )
    except Exception as logging_e:
        logger.error("Logging Error: " + str(logging_e))


def write_external_service_data(logger, response, editor_name):
    try:
        url_path = (
            response.request.url.scheme
            + "://"
            + response.request.url.host
            + response.request.url.path
        )
        logger.info(
            "[Editor] " + response.request.method + " " + url_path,
            extra={
                "extra_info": get_external_service_extra_info(response, editor_name)
            },
        )
    except Exception as logging_e:
        logger.error("Logging Error write_external_service_data: " + str(logging_e))



import io
import os
from pathlib import Path

import msoffcrypto
from dotenv import load_dotenv
from openpyxl import Workbook, load_workbook

load_dotenv()


def read_password_protected_excel(file_rel_path: Path) -> Workbook:
    decrypted_workbook = io.BytesIO()

    with open(file_rel_path, "rb") as file:
        office_file = msoffcrypto.OfficeFile(file)
        office_file.load_key(password=os.environ.get("EXCEL_PASSWORD"))
        office_file.decrypt(decrypted_workbook)

    # `filename` can also be a file-like object.
    workbook = load_workbook(filename=decrypted_workbook)

    return workbook



import random
from datetime import datetime, timedelta


def get_mock_managed_meeting_points(editor):
    if editor.name == "RDV360":
        return [
            {
                "id": "201",
                "name": "Mairie ANNEXE LILLE-SECLIN",
                "longitude": 3.0348016639327,
                "latitude": 50.549140395451,
                "_internal_id": "123123",
                "public_entry_address": "89 RUE ROGER BOUVRY",
                "zip_code": "59113",
                "city_name": "LILLE-SECLIN",
                "website": "https://www.ville-seclin.fr",
                "city_logo": "https://www.ville-seclin.fr/images/logo-ville-seclin/logo_ville_de_seclin.png",
                "_editor_name": "RDV360",
            },
            {
                "id": "202",
                "name": "Mairie de Quartier de Lille-Sud",
                "longitude": 3.0475818403133,
                "latitude": 50.612875943839,
                "internal_id": "456456",
                "public_entry_address": "83 Rue du Faubourg des Postes",
                "zip_code": "59000",
                "city_name": "LILLE-SECLIN",
                "website": "https://www.lille.fr/Lille-Sud2/Mairie-de-quartier-de-Lille-Sud",
                "city_logo": "https://www.ville-seclin.fr/images/logo-ville-seclin/logo_ville_de_seclin.png",
                "_editor_name": "RDV360",
            },
            {
                "id": "203",
                "name": "Mairie du 10e Arrondissement de Paris",
                "longitude": 2.357828,
                "latitude": 48.8717442,
                "_internal_id": "789879",
                "public_entry_address": "72 Rue du Faubourg Saint-Martin",
                "zip_code": "75010",
                "city_name": "Paris",
                "website": "https://mairie10.paris.fr",
                "city_logo": "https://www.grapheine.com/wp-content/uploads/Plan-de-travail-36paris-logo.jpg",
                "_editor_name": "RDV360",
            },
        ]
    elif editor.name == "OrionRDV":
        return [
            {
                "id": "FF65E",
                "name": "Mairie du 15e Arrondissement de Paris",
                "longitude": 2.2981566,
                "latitude": 48.8414021,
                "_internal_id": "12412315124",
                "public_entry_address": "31 Rue Péclet",
                "zip_code": "75015",
                "city_name": "Paris",
                "website": "https://mairie15.paris.fr",
                "city_logo": "https://www.grapheine.com/wp-content/uploads/Plan-de-travail-36paris-logo.jpg",
                "_editor_name": "OrionRDV",
            },
            {
                "id": "FF33E",
                "name": "Mairie du 18e Arrondissement de Paris",
                "longitude": 2.3303431,
                "latitude": 48.8909042,
                "_internal_id": "67868",
                "public_entry_address": "1 Pl. Jules Joffrin",
                "zip_code": "75018",
                "city_name": "Paris",
                "website": "https://mairie18.paris.fr",
                "city_logo": "https://www.grapheine.com/wp-content/uploads/Plan-de-travail-36paris-logo.jpg",
                "_editor_name": "OrionRDV",
            },
            {
                "id": "FF22E",
                "name": "Mairie du 2e Arrondissement de Paris",
                "longitude": 2.3402125,
                "latitude": 48.866407,
                "_internal_id": "12123345",
                "public_entry_address": "8 Rue de la Banque",
                "zip_code": "75002",
                "city_name": "Paris",
                "website": "https://mairiepariscentre.paris.fr/",
                "city_logo": "https://www.grapheine.com/wp-content/uploads/Plan-de-travail-36paris-logo.jpg",
                "_editor_name": "OrionRDV",
            },
        ]
    elif editor.name == "Ytop":
        return [
            {
                "id": "00342",
                "name": "Mairie du 13e Arrondissement de Paris",
                "longitude": 2.336658,
                "latitude": 48.8486635,
                "_internal_id": "998778",
                "public_entry_address": "1 Pl. d'Italie",
                "zip_code": "75013",
                "city_name": "Paris",
                "website": "https://mairie13.paris.fr/",
                "city_logo": "https://www.grapheine.com/wp-content/uploads/Plan-de-travail-36paris-logo.jpg",
                "_editor_name": "SynBird",
            },
        ]
    else:
        return []


def get_mock_slots(meeting_point, start_date=None, end_date=None):
    slots_list = []
    index = 0
    while index < 50:
        slots_list.append(
            {
                "callback_url": meeting_point["website"] + "/rendez-vous/passeports",
                "datetime": datetime.now().replace(
                    hour=(random.randint(7, 19)), minute=(random.randint(0, 2) * 20)
                )
                + timedelta(days=50 + random.randint(1, 100)),
            }
        )
        if slots_list[-1]["datetime"].weekday() == 6:
            slots_list.pop()
        index += 1

    # filter by dates

    if start_date and end_date:
        filtered_slots_list = []
        start_datetime = datetime.now().replace(
            year=start_date.year,
            month=start_date.month,
            day=start_date.day,
            hour=0,
            minute=0,
            second=0,
        )
        end_datetime = datetime.now().replace(
            year=end_date.year,
            month=end_date.month,
            day=end_date.day,
            hour=23,
            minute=59,
            second=59,
        )

        for slot in slots_list:
            if start_datetime < slot["datetime"] and end_datetime > slot["datetime"]:
                filtered_slots_list.append(slot)
        return filtered_slots_list, None
    else:
        return slots_list, None


def get_mock_slots_multiple(meeting_points, start_date=None, end_date=None):
    slots = {}
    for meeting_point in meeting_points:
        meeting_point_slots = [
            x[0] for x in get_mock_slots(meeting_point, start_date, end_date) if x
        ]
        slots[meeting_point["_internal_id"]] = meeting_point_slots
    print("Slots : " + str(len(slots)))
    return slots, None


def get_mock_slots_with_meeting_point_data(editors_list):
    meeting_points_list = []
    for editor in editors_list:
        meeting_points_list += get_mock_managed_meeting_points(editor)
    for meeting_point in meeting_points_list:
        slots_list = get_mock_slots(meeting_point)
        meeting_point["available_slots"] = slots_list
        meeting_point["distance_km"] = random.uniform(0.3, 10)
    return meeting_points_list


def get_mock_applications(application_ids):
    result = {}
    for application_id in application_ids:
        result[application_id] = [
            {
                "meeting_point": "Mairie du 2e Arrondissement de Paris",
                "datetime": "2022-11-10T10:00Z",
                "management_url": "https://mairiepariscentre.paris.fr/rendez-vous/predemande?num=6123155111",
                "cancel_url": "https://mairiepariscentre.paris.fr/rendez-vous/annulation?num=6123155111",
            },
            {
                "meeting_point": "Mairie du 13e Arrondissement de Paris",
                "datetime": "2022-11-10T16:00Z",
                "management_url": "https://mairie13.paris.fr/rendez-vous/predemande?num=6123155111",
                "cancel_url": "https://mairiepariscentre.paris.fr/rendez-vous/annulation?num=6123155111",
            },
        ]
    return result



import json
import logging
from datetime import datetime, timedelta

from dateutil.relativedelta import relativedelta
from fastapi import APIRouter, HTTPException, WebSocket, WebSocketDisconnect

from ...db.utils import add_ws_use_rates, get_all_editors, get_ws_use_rates
from ...services.data_validator import is_valid_search_criteria
from ...services.search_time_slots import search_slots

logging.getLogger("uvicorn.error").setLevel(logging.ERROR)

router = APIRouter()


@router.websocket("/SlotsFromPositionStreaming")
async def slots_from_position_streaming(websocket: WebSocket):
    distance = [0.99, 5, 10, 15, 20, 25, 30, 100]
    index = 0
    _logger = logging.getLogger("root")
    try:
        await websocket.accept()
        while True:
            raw_data = await websocket.receive_text()
            # Check use rate per minute
            if get_ws_use_rates(websocket.client.host) > 30:
                await websocket.send_text("end_of_search")
                raise Exception("Websocket rate limit exceeded")
            else:
                add_ws_use_rates(websocket.client.host)
            start_time = datetime.now()
            data = json.loads(raw_data)

            if "department_code" in data:
                data["radius_km"] = 100
                if not is_valid_search_criteria(
                    search_criteria=data, search_by_department=True
                ):
                    raise HTTPException(
                        status_code=400, detail="Invalid search criteria"
                    )
            else:
                if not is_valid_search_criteria(data):
                    raise HTTPException(
                        status_code=400, detail="Invalid search criteria"
                    )
            latitude = float(data["latitude"])
            longitude = float(data["longitude"])
            start_date = None
            try:
                if "start_date" in data:
                    start_date = datetime.strptime(
                        data["start_date"], "%Y-%m-%d"
                    ).date()
            except ValueError:
                pass
            end_date = None
            try:
                if "end_date" in data:
                    end_date = datetime.strptime(data["end_date"], "%Y-%m-%d").date()
            except ValueError:
                pass
            radius_km = float(data["radius_km"])
            reason = None
            if "reason" in data:
                reason = data["reason"]
            documents_number = None
            if "documents_number" in data:
                documents_number = int(data["documents_number"])

            settings = {}
            result = []

            if "smart_search" in data and data["smart_search"]:
                months = 1
                valide_slots = False
                while radius_km <= 30 and not valide_slots:
                    index = distance.index(radius_km)
                    result, errors = await search_slots(
                        longitude,
                        latitude,
                        start_date,
                        start_date + relativedelta(months=months),
                        radius_km,
                        reason,
                        documents_number,
                        websocket=websocket,
                        department_code=(
                            "department_code" in data and data["department_code"]
                        ),
                    )
                    if result:
                        for meeting_point in result:
                            valide_slots = is_valide_slots(meeting_point)
                            if valide_slots:
                                break

                    months += 1
                    radius_km = distance[index + 1]
                settings["radius"] = distance[index]
                settings["end_date"] = start_date + relativedelta(months=months - 1)

            else:
                result, errors = await search_slots(
                    longitude,
                    latitude,
                    start_date,
                    end_date,
                    radius_km,
                    reason,
                    documents_number,
                    websocket=websocket,
                    department_code=(
                        "department_code" in data and data["department_code"]
                    ),
                )

            json_text = json.dumps(
                {
                    "step": "end_of_search",
                    "editors_number": len(get_all_editors()),
                    "editor_errors_number": len(errors),
                    "settings": settings,
                },
                default=str,
            )
            await websocket.send_text(json_text)
            _logger.info(
                "End of websocket search",
                extra={
                    "extra_info": {
                        "type": "access",
                        "searchCriteria": data,
                        "searchLocation": {"lat": latitude, "lon": longitude},
                        "response_time": (datetime.now() - start_time).microseconds
                        / 1000,
                        "protocol": "websocket",
                        "realip": websocket.client.host,
                    }
                },
            )
    except WebSocketDisconnect:
        _logger.debug("Client disconnected.")
    except HTTPException as websocket_e:
        _logger.error("HTTP Error during websocket connexion : %s", websocket_e.detail)
    except Exception as websocket_e:
        _logger.error("Error during websocket connexion : %s", websocket_e)


def is_valide_slots(meeting_point):
    for slot in meeting_point["available_slots"]:
        if datetime_from_str(slot["datetime"]) >= datetime.now():
            return True
    return False


def datetime_from_str(datetime_str):
    datetime_datetime = None
    for datetime_format in [
        "%Y-%m-%dT%H:%MZ",
        "%Y-%m-%dT%H:%M:%S%z",
        "%Y-%m-%dT%H:%M:%S",
    ]:
        try:
            datetime_datetime = datetime.strptime(datetime_str, datetime_format)
            if datetime_datetime.tzinfo:
                utcoffset = datetime_datetime.utcoffset().total_seconds() / 60 / 60
                datetime_datetime = datetime_datetime.replace(tzinfo=None) + timedelta(
                    hours=utcoffset
                )
            break
        except ValueError:
            pass

    return datetime_datetime



import logging
import os
from functools import lru_cache
from typing import Set

from pydantic import BaseSettings, Field, validator


class Settings(BaseSettings):
    debug: bool
    api_prefix: str = "/api"
    project_name: str = "HUB RDV"

    logging_level: int = 0
    loggers: Set[str] = {"uvicorn.asgi", "uvicorn.access"}

    editors_list = []
    meeting_point_list = []
    offline_meeting_point_list = []
    ws_use_rates = {}
    database_url: str = Field(..., env="DATABASE_URL")

    class Config:
        env_file = ".env"
        env_file_encoding = "utf-8"

    @validator("debug")
    def set_name(cls, debug):
        return debug or False


@lru_cache()
def get_settings() -> Settings:
    debug = os.environ.get("ENVIRONMENT") not in ["dev", "preprod"]
    settings = Settings(debug=debug)
    settings.logging_level = logging.DEBUG if debug else logging.INFO
    return settings



from sqlalchemy import Column, String

from ..db.postgresdb import Base
from ..utils.departments import find_department
from ..utils.utils import cannonize


class CertifiedMunicipality(Base):
    """
    Represents a Certified Municipality
    """

    __tablename__ = "certified_municipality"

    ugf = Column(String, primary_key=True, index=True)
    town_hall_name = Column(String)
    address = Column(String)
    zip_code = Column(String)
    phone_number = Column(String)
    website = Column(String)
    city_name = Column(String)
    appointment_details = Column(String)
    service_opening_date = Column(String)
    label = Column(String)

    def complete(self):
        department_code, department_name = find_department(self.zip_code)
        cannonized_name = cannonize(self.city_name)

        return {
            "ugf": self.ugf,
            "town_hall_name": self.town_hall_name,
            "address": self.address,
            "zip_code": self.zip_code,
            "phone_number": self.phone_number,
            "website": self.website,
            "city_name": self.city_name,
            "appointment_details": self.appointment_details,
            "service_opening_date": self.service_opening_date,
            "label": self.label,
            "department_code": department_code,
            "department_name": department_name,
            "cannonized_name": cannonized_name,
        }



"""
This module contains Pydantic schemas for database models related to token and editor entities.
It includes the following schemas:
- TokenAntsToEditorBase
- TokenAntsToEditorCreate
- TokenAntsToEditor
- TokenEditorToAntsBase
- TokenEditorToAntsCreate
- TokenEditorToAnts
- EditorBase
- EditorCreate
- EditorUpdate
- Editor
"""
import datetime

from pydantic import BaseModel


class TokenAntsToEditorBase(BaseModel):
    """
    Base schema for token issued by ANTS to editor
    """

    id: int
    id_editor: int
    token: str
    validity_date: datetime

    class Config:
        orm_mode = True


class TokenAntsToEditorCreate(TokenAntsToEditorBase):
    """
    Schema for creating token issued by ANTS to editor
    """


class TokenAntsToEditor(TokenAntsToEditorBase):
    """
    Schema for token issued by ANTS to editor
    """


class TokenEditorToAntsBase(BaseModel):
    """
    Base schema for token issued by editor to ANTS
    """

    id: int
    id_editor: int
    token: str
    validity_date: datetime

    class Config:
        orm_mode = True


class TokenEditorToAntsCreate(TokenEditorToAntsBase):
    """
    Schema for creating token issued by editor to ANTS
    """


class TokenEditorToAnts(TokenEditorToAntsBase):
    """
    Schema for token issued by editor to ANTS
    """


class EditorBase(BaseModel):
    """
    Base schema for editor
    """

    name: str
    url: str
    header_name: str


class EditorCreate(EditorBase):
    """
    Schema for creating editor
    """


class EditorUpdate(EditorBase):
    """
    Schema for updating editor
    """


class Editor(EditorBase):
    """
    Schema for editor
    """

    id: int

    class Config:
        orm_mode = True


class EditorPanoramaBase(BaseModel):
    """
    Base schema for editor_panorama
    """

    name: str
    logo_url: str
    site_url: str
    contact_url: str
    overseas: bool
    integration_predemande: bool
    limit_appointment_outside: bool
    saas: bool
    use_queue: bool
    is_connect_to_hub: bool
    is_connect_to_anti_duplication: bool


class EditorPanoramaCreate(EditorPanoramaBase):
    """
    Schema for creating editor
    """


class EditorPanoramaUpdate(EditorPanoramaBase):
    """
    Schema for updating editor
    """


class EditorPanorama(EditorPanoramaBase):
    """
    Schema for editor
    """

    id: int

    class Config:
        orm_mode = True



from sqlalchemy import Boolean, Column, Integer, String

from ..db.postgresdb import Base


class EditorPanorama(Base):
    __tablename__ = "editor_panorama"
    id = Column(Integer, primary_key=True)
    name = Column(String, nullable=True)
    logo_url = Column(String, nullable=True)
    site_url = Column(String, nullable=True)
    contact_url = Column(String, nullable=True)
    overseas = Column(Boolean)
    integration_predemande = Column(Boolean)
    limit_appointment_outside = Column(Boolean)
    saas = Column(Boolean)
    use_queue = Column(Boolean)
    is_connect_to_hub = Column(Boolean)
    is_connect_to_anti_duplication = Column(Boolean)



import json
import logging
from datetime import datetime, timezone


def get_app_log(record):
    json_obj = {
        "log.level": record.levelname,
        "type": "app",
        "@timestamp": datetime.now(timezone.utc).strftime("%Y-%m-%dT%H:%M:%S.%f%z"),
        "message": record.message,
    }

    if hasattr(record, "extra_info"):
        for key in record.extra_info:
            json_obj[key] = record.extra_info[key]

    return json_obj


def get_access_log(record):
    json_obj = {
        "log.level": record.levelname,
        "type": "access",
        "@timestamp1": datetime.strptime(
            record.asctime, "%Y-%m-%d %H:%M:%S,%f"
        ).strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3]
        + "Z",
        "@timestamp": datetime.now(timezone.utc).strftime("%Y-%m-%dT%H:%M:%S.%f%z"),
        "message": record.message,
        "response_time": record.extra_info["response_time"],
        "protocol": record.extra_info["protocol"],
    }
    for key in record.extra_info:
        json_obj[key] = record.extra_info[key]
    return json_obj


class CustomFormatter(logging.Formatter):
    def __init__(self, formatter):
        logging.Formatter.__init__(self, formatter)

    def format(self, record):
        logging.Formatter.format(self, record)
        if not hasattr(record, "extra_info") or record.extra_info["type"] == "app":
            return json.dumps(get_app_log(record), ensure_ascii=False)
        else:
            return json.dumps(get_access_log(record), ensure_ascii=False)



from fastapi import APIRouter

from ..controllers.routes import data, external, internal, time_slots, websocket

router = APIRouter()


router.include_router(external.router, tags=["API interfacées avec les éditeurs"])
router.include_router(
    time_slots.router, tags=["Annexe: Accès aux rendez-vous disponibles"]
)
router.include_router(internal.router, tags=["Interne ANTS"])
router.include_router(websocket.router, tags=["Websocket"])
router.include_router(data.router, tags=["API de gestion des données"])



from typing import List, Optional

from pydantic import BaseModel

from ..models.available_time_slot import AvailableTimeSlot


class Municipality(BaseModel):
    id: str
    name: str
    longitude: float
    latitude: float
    _internal_id: str
    public_entry_address: str
    zip_code: str
    city_name: str
    decoded_city_name: Optional[str]
    website: Optional[str]
    city_logo: Optional[str]
    _editor_name: Optional[str]

    class Config:
        underscore_attrs_are_private = True


class MunicipalityWithDistance(Municipality):
    distance_km: float


class MunicipalityWithSlots(MunicipalityWithDistance):
    available_slots: List[AvailableTimeSlot]


class OfflineMunicipality(BaseModel):
    id: str
    municipality: str
    # longitude: float
    # latitude: float
    public_entry_address: str
    zip_code: str
    phone_number: Optional[str]
    decoded_city_name: Optional[str]
    url: Optional[str]
    logo: Optional[str]
    website: Optional[str]


class OfflineMunicipalityWithDistance(OfflineMunicipality):
    distance: float



"""
This module contains the code for creating a PostgreSQL database connection and session.

Functions:
- check_database: checks if the database exists, and creates it if it doesn't.
"""

from sqlalchemy import create_engine
from sqlalchemy.orm import declarative_base, scoped_session, sessionmaker
from sqlalchemy_utils import create_database, database_exists

from ..core.config import get_settings


def check_database():
    """
    Checks if the database exists, and creates it if it doesn't.
    """
    if not database_exists(get_settings().database_url):
        create_database(get_settings().database_url)


engine = create_engine(
    get_settings().database_url,
    pool_size=5,
    max_overflow=2,
    echo=False,
    pool_pre_ping=True,
)

session: scoped_session = scoped_session(
    sessionmaker(autocommit=False, autoflush=False, bind=engine)
)

Base = declarative_base()



import io
import json
import logging
from typing import List

import xlrd
from fastapi import UploadFile
from sqlalchemy.orm import sessionmaker

from src.hubrdvmairie.models.meeting_point import MeetingPoint

from ..crud.crud_meeting_point import meetingPoint as crud

logger = logging.getLogger(__name__)


def get_all(session) -> List[MeetingPoint]:
    try:
        return crud.get_all(session)
    except Exception as e:
        logger.error("Error while getting all meeting points : %s", str(e))


def get_by_ugf(session, ugf: str) -> MeetingPoint:
    try:
        return crud.get_by_ugf(session, ugf=ugf)
    except Exception as e:
        logger.error("Error while getting meeting point by ugf : %s", str(e))


async def update_meeting_points_table(session: sessionmaker, uploaded_file: UploadFile):
    meeting_points = await read_meeting_point_from_file_streaming(uploaded_file)
    create_list = []
    unchanged_list = []
    nb_meeting_points = len(meeting_points)
    for meeting_point in meeting_points:
        res = crud.save_or_update(session, obj_in=meeting_point)
        if res[0] == "created":
            create_list.append(res[1])
        else:
            unchanged_list.append(res[1])

    yield json.dumps(
        {
            "nb_meeting_points": nb_meeting_points,
            "created : ": str(len(create_list)),
            "unchanged : ": str(len(unchanged_list)),
        }
    )


async def read_meeting_point_from_file_streaming(
    uploaded_file: UploadFile,
) -> MeetingPoint:
    # read file depending on its type
    if uploaded_file.filename.endswith(".xlsx"):
        return await read_meeting_points_file_streaming(uploaded_file)
    else:
        raise TypeError("Unknown file type : " + str(uploaded_file.filename))


async def read_meeting_points_file_streaming(uploaded_file: UploadFile):
    meeting_points = set()

    file_content = await uploaded_file.read()
    xls_data = io.BytesIO(file_content)

    workbook = xlrd.open_workbook(file_contents=xls_data.read())
    worksheet = workbook.sheet_by_index(2)

    for row_idx in range(1, worksheet.nrows):
        row = worksheet.row(row_idx)
        ugf = str(int(row[0].value))
        editor_name_and_id = row[1].value
        city_name = row[2].value
        id_editor = row[3].value

        meeting_point = MeetingPoint(
            ugf=ugf,
            editor_name_and_id=editor_name_and_id,
            city_name=city_name,
            id_editor=id_editor,
        )
        meeting_points.add(meeting_point)

    return meeting_points



import json
from typing import Any

from fastapi import APIRouter, Depends, File, HTTPException, Request, UploadFile
from fastapi.responses import JSONResponse, StreamingResponse
from pydantic import Required
from slowapi import Limiter
from slowapi.util import get_remote_address
from sqlalchemy.orm import sessionmaker

from ...core.custom_validation_error import CustomValidationError
from ...db.postgresdb_utils import get_database
from ...services.certified_municipality_service import (
    get_all_certified_municipalities,
    update_certified_municipality_table,
)
from ...services.meeting_point_service import get_all, update_meeting_points_table
from ..dependencies.auth_token import verify_internal_auth_token

router = APIRouter()

limiter = Limiter(key_func=get_remote_address)


@router.get(
    "/meetingPoints",
    responses={
        200: {
            "description": "Liste des meeting points en base de données",
            "content": {
                "application/json": {
                    "example": [
                        {
                            "city_name": "Corme-Royal",
                            "id_editor": 5,
                            "editor_name_and_id": "RDV360100358",
                            "ugf": "17046",
                        },
                        {
                            "city_name": "Huelgoat",
                            "id_editor": 2,
                            "editor_name_and_id": "Synbird2036",
                            "ugf": "29014",
                        },
                    ]
                }
            },
        }
    },
    dependencies=[Depends(verify_internal_auth_token)],
)
@limiter.limit("30/minute")
async def get_meeting_points(
    request: Request, session: sessionmaker = Depends(get_database)
) -> Any:
    """
    Récupère les données des meeting points en base de données.
    """
    try:
        return get_all(session)
    except Exception as e:
        raise HTTPException(
            status_code=500,
            detail=f"Error while getting meeting points : {str(e)}",
        )
    finally:
        session.close()


@router.post(
    "/updateMeetingPoints",
    responses={
        200: {
            "description": "Mise à jour des meeting points effectuée avec succès",
            "content": {
                "application/json": {
                    "example": {
                        "nb_meeting_points": 3043,
                        "created : ": "0",
                        "unchanged : ": "3043",
                    }
                }
            },
        }
    },
    dependencies=[Depends(verify_internal_auth_token)],
)
async def update_meeting_points(
    request: Request,
    uploaded_file: UploadFile = File(
        default=Required, media_type="application/vnd.ms-excel"
    ),
    session: sessionmaker = Depends(get_database),
) -> Any:
    """
    <p>
        L’API permet à l'ANTS de mettre à jour la table des meeting points.
        Le fichier Excel doit être sous la forme suivante:
        <table class="table-bordered">
            <thead>
                <tr>
                    <th>UGF/th>
                    <th>editor_name_and_id</th>
                    <th>city_name</th>
                    <th>city_name</th>
                </tr>
            </thead>
            <tbody>
                <tr>
                    <td>{Num UGF}</td>
                    <td>{Nom éditeur concaténé avec leur id meeting point}</td>
                    <td>{Nom de la ville}</td>
                    <td>{ID de l'éditeur}</td>
                </tr>
                <tr>
                    <td>{Num UGF}</td>
                    <td>{Nom éditeur concaténé avec leur id meeting point}</td>
                    <td>{Nom de la ville}</td>
                    <td>{ID de l'éditeur}</td>
                </tr>
            </tbody>
        </table>
        </br></br>Accès via un token de sécurité unique.
    </p>
    <p>
    Règle de gestion :</br>
    <ul>
        <li>Si l'ensemble ugf et editor_name_and_id n'existe pas: un meeting point est créé en base.</li>
        <li>Sinon on le comptabilise dans les inchangés.</li>
    </ul>
    </p>
    """
    try:
        return StreamingResponse(
            content=update_meeting_points_table(session, uploaded_file),
            media_type="application/json",
        )
    except Exception:
        raise HTTPException(
            status_code=500,
            detail="Error while updating meeting points",
        )
    finally:
        session.close()


@router.get(
    "/certifiedMunicipalities",
    responses={
        200: {
            "description": "Liste des mairies certifiées en base de données",
            "content": {
                "application/json": {
                    "example": [
                        {
                            "department_code": "69",
                            "department_name": "Rhône",
                            "municipalities": [
                                {
                                    "ugf": "69080",
                                    "town_hall_name": "Mairie de COLOMBIER SAUGNIEU",
                                    "address": "14 rue de la Mairie",
                                    "zip_code": "69124",
                                    "phone_number": "04 78 32 80 17",
                                    "website": "https://www.mairie-colombiersaugnieu.fr/demarches-administratives/en-mairie/",
                                    "city_name": "COLOMBIER SAUGNIEU",
                                    "appointment_details": "Lundi : 14h – 17h\nMardi et jeudi : 9h – 12h / 14h – 18h\nMercredi : 9h – 17h\nVendredi et samedi : 9h – 12h ",
                                    "service_opening_date": "2024-03-25",
                                    "label": "sans rendez-vous",
                                    "department_code": "69",
                                    "department_name": "Rhône",
                                    "cannonized_name": "colombiersaugnieu",
                                },
                                {
                                    "ugf": "69071",
                                    "town_hall_name": "Mairie de CHASSIEU",
                                    "address": "60 rue de la République",
                                    "zip_code": "69680",
                                    "phone_number": "04 72 05 44 00",
                                    "website": "https://www.chassieu.fr/",
                                    "city_name": "CHASSIEU",
                                    "appointment_details": "Du lundi au vendredi de 08h00 à 12h00 et de 13h00 à 17h00,\nle mardi de 08h00 à 12h00 et de 14h00 à 17h00\npendant les permanences de la direction (hors vacances scolaires) :\nle mardi de 17h00 à 19h00,\nle samedi de 09h00 à 12h00",
                                    "service_opening_date": "2024-03-11",
                                    "label": "sans rendez-vous",
                                    "department_code": "69",
                                    "department_name": "Rhône",
                                    "cannonized_name": "chassieu",
                                },
                            ],
                        },
                        {
                            "department_code": "92",
                            "department_name": "Hauts-de-Seine",
                            "municipalities": [
                                {
                                    "ugf": "92040",
                                    "town_hall_name": "Mairie Annexe de CLAMART",
                                    "address": "5, rue d’Auvergne",
                                    "zip_code": "92140",
                                    "phone_number": "01 46 62 35 35",
                                    "website": "https://www.clamart.fr",
                                    "city_name": "Clamart",
                                    "appointment_details": " Les lundis, mercredis, vendredis de 8h30 à 12h30 et de 13h30 à 17h00",
                                    "service_opening_date": "2024-03-11",
                                    "label": "sans rendez-vous",
                                    "department_code": "92",
                                    "department_name": "Hauts-de-Seine",
                                    "cannonized_name": "clamart",
                                },
                            ],
                        },
                    ]
                }
            },
        }
    },
)
async def get_certified_municipalities(
    request: Request, session: sessionmaker = Depends(get_database)
) -> Any:
    """
    Récupère les données des mairies certifiées en base de données.
    """
    try:
        return get_all_certified_municipalities(session)
    except Exception as e:
        raise HTTPException(
            status_code=500,
            detail=f"Error while getting certified municipality : {str(e)}",
        )
    finally:
        session.close()


@router.post(
    "/updateCertifiedMunicipalities",
    responses={
        200: {
            "description": "Mise à jour des mairies certifiées effectuée avec succès",
            "content": {
                "application/json": {
                    "example": {
                        "nb_certified_municipality": 12,
                        "created : ": "12",
                        "unchanged : ": "0",
                    }
                }
            },
        },
        422: {
            "description=": "Erreur de validation -> Fichier invalide",
            "content": {
                "application/json": {
                    "example": [
                        {
                            "detail": [
                                "ligne n°639 : ('Mairie de LONGJUMEAU', 91019, '6 Bis Rue Leontine Sohier', 91160, 'LONGJUMEAU', '01 64 54 19 00', 'https://www.longjumeau.fr   Hôtel de ville 6 bis rue Léontine Sohier 91160 Longjumeau Lundi, jeudi et vendredi : 8h30 à 12h30 et 13h30 à 18h. Mardi : 8h30 à 12h30 et de 14h30 à 18h. Mercredi et samedi : 8h30 à 12h. Fermé : le mercredi après-midi et le samedi après-midi, le dimanche', \"Pour cette démarche, la mairie de LONGJUMEAU accueille sans rendez-vous. Vous pouvez vous y rendre sur les horaires d'ouverture habituels.\", '20/05/2024', 'Sans rendez-vous') - erreur : 1 validation error for CertifiedMunicipality\nwebsite\n  ensure this value has at most 255 characters (type=value_error.any_str.max_length; limit_value=255)",
                                "ligne n°644 : (None, None, None, None, None, None, None, None, None, None) - erreur : Cette ligne est vide, merci de la supprimer",
                                "ligne n°645 : (None, None, None, None, None, None, None, None, None, None) - erreur : Cette ligne est vide, merci de la supprimer",
                                "ligne n°646 : (None, None, None, None, None, None, None, None, None, None) - erreur : Cette ligne est vide, merci de la supprimer",
                            ]
                        }
                    ]
                }
            },
        },
    },
    dependencies=[Depends(verify_internal_auth_token)],
)
async def update_certified_municipalities(
    request: Request,
    uploaded_file: UploadFile = File(
        default=Required, media_type="application/vnd.ms-excel"
    ),
    session: sessionmaker = Depends(get_database),
) -> Any:
    """
    <p>
        L’API permet à l'ANTS de mettre à jour la table des mairies certifiées.
        Le fichier Excel doit être sous la forme suivante:
        <table class="table-bordered">
            <thead>
                <tr>
                    <th>Nom de la mairie</th>
                    <th>UGF</th>
                    <th>Adresse</th>
                    <th>Code postal</th>
                    <th>Ville</th>
                    <th>Téléphone</th>
                    <th>Site internet</th>
                    <th>Modalités de rendez-vous</th>
                    <th>Date d'ouverture du service</th>
                    <th>Label</th>
                </tr>
            </thead>
            <tbody>
                <tr>
                    <td>Mairie de CHASSIEU</td>
                    <td>69071</td>
                    <td>60 rue de la République</td>
                    <td>69680</td>
                    <td>CHASSIEU</td>
                    <td>04 72 05 44 00</td>
                    <td>https://www.chassieu.fr/</td>
                    <td>Du lundi au vendredi de 08h00 à 12h00 et de 13h00 à 17h00...</td>
                    <td>11/03/2024</td>
                    <td>sans rendez-vous</td>
                </tr>
                <tr>
                    <td>Mairie principale de CLAMART</td>
                    <td>92072</td>
                    <td>1, place Maurice Gunsbourg</td>
                    <td>92140</td>
                    <td>Clamart</td>
                    <td>01 46 62 35 35</td>
                    <td>https://www.clamart.fr</td>
                    <td>les lundis, mercredis, jeudis de 8h30 à 12h30 ...</td>
                    <td>11/03/2024</td>
                    <td>sans rendez-vous</td>
                </tr>
            </tbody>
        </table>
        </br></br>Accès via un token de sécurité unique.
    </p>
    <p>
    Règle de gestion :</br>
    <ul>
        <li>Si l'ugf n'existe pas: une entrée est créé en base.</li>
        <li>Sinon on le comptabilise dans les inchangés.</li>
    </ul>
    Il est important que chaque champ soit renseigné avec le bon format (url commençant par https:// etc...).
    </p>
    """
    try:
        response_data = await update_certified_municipality_table(
            session, uploaded_file
        )
        return JSONResponse(
            content=json.loads(response_data),
            media_type="application/json",
        )
    except CustomValidationError as e:
        raise HTTPException(
            status_code=422,
            detail=e.errors,
        )
    except Exception as e:
        raise HTTPException(
            status_code=500,
            detail=f"Error while updating meeting points: {str(e)}",
        )
    finally:
        session.close()



from typing import TYPE_CHECKING

from sqlalchemy import Column, ForeignKey, Integer, String
from sqlalchemy.orm import relationship

from ..db.postgresdb import Base

if TYPE_CHECKING:
    from .editor_model import Editor  # noqa: F401


class MeetingPoint(Base):
    """
    Represents a Meeting point
    """

    __tablename__ = "meeting_point"

    ugf = Column(String, primary_key=True, index=True)
    id_editor = Column(Integer, ForeignKey("editor.id"))
    editor_name_and_id = Column(String, primary_key=True, index=True, unique=True)
    city_name = Column(String, nullable=True)
    editor = relationship("Editor", back_populates="meeting_points")



from typing import Optional

from sqlalchemy.orm import Session

from ..crud.base import CRUDBase
from ..models.token_ants_to_editor import TokenAntsToEditor
from ..schemas.token_ants_to_editor import (
    TokenAntsToEditorCreate,
    TokenAntsToEditorUpdate,
)


class CRUDTokenAntsToEditor(
    CRUDBase[TokenAntsToEditor, TokenAntsToEditorCreate, TokenAntsToEditorUpdate]
):
    """
    CRUD operations for TokenAntsToEditor model.
    """

    def get_by_token(self, db: Session, *, token: str) -> Optional[TokenAntsToEditor]:
        """
        Retrieve a token by its value.

        Args:
            db (Session): The database session.
            token (str): The token value.

        Returns:
            Optional[TokenAntsToEditor]: The token if found, None otherwise.
        """
        return (
            db.query(TokenAntsToEditor).filter(TokenAntsToEditor.token == token).first()
        )

    def get_all(self, db: Session):
        """
        Returns all TokenAntsToEditor objects from the database.

        Args:
        - db: SQLAlchemy Session instance.

        Returns:
        - List of TokenAntsToEditor objects.
        """
        return db.query(TokenAntsToEditor).all()

    def get_by_editor_id(
        self, db: Session, *, editor_id: int
    ) -> Optional[TokenAntsToEditor]:
        """
        Retrieve a token for a given editor_id.

        Args:
            db (Session): The database session.
            editor_id (int): The ID of the editor.

        Returns:
            Optional[TokenAntsToEditor]: The token for the given editor_id, if it exists.
        """
        return (
            db.query(TokenAntsToEditor)
            .filter(TokenAntsToEditor.editor_id == editor_id)
            .first()
        )


token_ants_to_editor = CRUDTokenAntsToEditor(TokenAntsToEditor)



from typing import List

from sqlalchemy.orm import Session

from ..crud.base import CRUDBase
from ..models.certified_municipality import CertifiedMunicipality
from ..schemas.certified_municipality import (
    CertifiedMunicipalityCreate,
    CertifiedMunicipalityUpdate,
)


class CRUDCertifiedMunicipality(
    CRUDBase[
        CertifiedMunicipality, CertifiedMunicipalityCreate, CertifiedMunicipalityUpdate
    ]
):
    """
    CRUD operations for the CertifiedMunicipality model.
    """

    def get_all(self, db: Session) -> List[CertifiedMunicipality]:
        """
        Retrieve all CertifiedMunicipality.

        Args:
            db (Session): The database session.

        Returns:
            List[CertifiedMunicipality]: A list of all CertifiedMunicipality.
        """
        return db.query(CertifiedMunicipality).all()

    def update(
        self,
        db: Session,
        *,
        db_obj: CertifiedMunicipality,
        obj_in: CertifiedMunicipality
    ) -> CertifiedMunicipality:
        """
        Update a CertifiedMunicipality.
        """
        db_obj.ugf = obj_in.ugf
        db_obj.town_hall_name = obj_in.town_hall_name
        db_obj.address = obj_in.address
        db_obj.zip_code = obj_in.zip_code
        db_obj.phone_number = obj_in.phone_number
        db_obj.website = obj_in.website
        db_obj.appointment_details = obj_in.appointment_details
        db_obj.service_opening_date = obj_in.service_opening_date
        db.commit()
        return db_obj

    def save_or_update(
        self, db: Session, *, obj_in: CertifiedMunicipality
    ) -> tuple[str, CertifiedMunicipality]:
        """
        Save or update a CertifiedMunicipality.
        """
        certified_municipality = (
            db.query(CertifiedMunicipality)
            .filter(CertifiedMunicipality.ugf == obj_in.ugf)
            .first()
        )
        if certified_municipality:
            self.update(db, db_obj=certified_municipality, obj_in=obj_in)
            return "updated", certified_municipality.ugf
        else:
            self.create(db, obj_in=obj_in)
        return "created", obj_in.ugf


certified_municipality = CRUDCertifiedMunicipality(CertifiedMunicipality)



import io
import json
import logging
from collections import defaultdict
from typing import List, Set, Tuple

from fastapi import UploadFile
from openpyxl import load_workbook
from pydantic import ValidationError
from sqlalchemy.orm import sessionmaker

from src.hubrdvmairie.core.custom_validation_error import CustomValidationError
from src.hubrdvmairie.schemas.certified_municipality import CertifiedMunicipality

from ..crud.crud_certified_municipality import certified_municipality as crud

logger = logging.getLogger(__name__)


def get_all(session) -> List[CertifiedMunicipality]:
    try:
        return crud.get_all(session)
    except Exception as e:
        logger.error("Error while getting all certified_municipality : %s", str(e))


def get_all_certified_municipalities(
    session: sessionmaker,
) -> List[CertifiedMunicipality]:
    municipalities = get_all(session)
    certified_municipalities_complete = []
    for certified_municipality in municipalities:
        complete = certified_municipality.complete()
        certified_municipalities_complete.append(complete)
    departments = defaultdict(list)

    for municipality in certified_municipalities_complete:
        department_code = municipality["department_code"]
        departments[department_code].append(municipality)

    departments_list = []
    for department_code, municipalities in departments.items():
        department_info = {
            "department_code": department_code,
            "department_name": municipalities[0]["department_name"],
            "municipalities": municipalities,
        }
        departments_list.append(department_info)
    departments_list = sorted(departments_list, key=lambda x: x["department_code"])
    return departments_list


async def update_certified_municipality_table(
    session: sessionmaker, uploaded_file: UploadFile
):
    (
        certified_municipalities,
        errors,
    ) = await read_certified_municipality_from_file_streaming(uploaded_file)
    if errors:
        raise CustomValidationError(errors)
    else:
        create_list = []
        unchanged_list = []
        nb_certified_municipalities = len(certified_municipalities)
        for certified_municipality in certified_municipalities:
            res = crud.save_or_update(session, obj_in=certified_municipality)
            if res[0] == "created":
                create_list.append(res[1])
            else:
                unchanged_list.append(res[1])

        response_data = {
            "nb_certified_municipality": nb_certified_municipalities,
            "created": str(len(create_list)),
            "unchanged": str(len(unchanged_list)),
        }
        return json.dumps(response_data)


async def read_certified_municipality_from_file_streaming(
    uploaded_file: UploadFile,
) -> CertifiedMunicipality:
    # read file depending on its type
    if uploaded_file.filename.endswith(".xlsx"):
        return await read_certified_municipality_file_streaming(uploaded_file)
    else:
        raise TypeError("Unknown file type : " + str(uploaded_file.filename))


async def read_certified_municipality_file_streaming(
    uploaded_file: UploadFile,
) -> Tuple[Set["CertifiedMunicipality"], list]:
    certified_municipalities = set()
    errors = []

    file_content = await uploaded_file.read()
    xls_data = io.BytesIO(file_content)
    data = load_workbook(xls_data, read_only=True).active

    for i, row in enumerate(
        data.iter_rows(min_row=2, max_row=data.max_row, values_only=True), start=2
    ):
        row_data = {
            "town_hall_name": row[0],
            "ugf": str(row[1]),
            "address": row[2],
            "zip_code": str(row[3]),
            "city_name": row[4],
            "phone_number": row[5],
            "website": row[6],
            "appointment_details": str(row[7]),
            "service_opening_date": str(row[8]),
            "label": row[9],
        }

        if all(value in ["None", None] for value in row_data.values()):
            errors.append(
                f"ligne n°{i} : {row} - erreur : Cette ligne est vide, merci de la supprimer"
            )
            continue

        if any(value in ["None", None] for value in row_data.values()):
            errors.append(f"ligne n°{i} : {row} - erreur : au moins un champ est vide")
            continue

        try:
            certified_municipality = CertifiedMunicipality(
                ugf=row_data["ugf"],
                town_hall_name=row_data["town_hall_name"],
                address=row_data["address"],
                zip_code=row_data["zip_code"],
                city_name=row_data["city_name"],
                phone_number=row_data["phone_number"],
                website=row_data["website"],
                appointment_details=row_data["appointment_details"],
                service_opening_date=row_data["service_opening_date"],
                label=row_data["label"],
            )
            CertifiedMunicipality.parse_obj(certified_municipality)
            certified_municipalities.add(certified_municipality)
        except ValidationError as e:
            logger.error(
                "Error while reading certified_municipality from file : %s", str(e)
            )
            errors.append(f"ligne n°{i} : {row} - erreur : {str(e)}")

    return certified_municipalities, errors



from datetime import datetime, timedelta


def isfloat(num):
    """validate the value is float or

    Args:
        num (Any): number

    Returns:
        bool: is float or not
    """
    try:
        float(num)
        return True
    except ValueError:
        return False


def is_valid_search_criteria(search_criteria, search_by_department=False):
    """function that accept dict object to validated every value of this object

    Args:
        search_criteria (dict): object contain as keys radius_km, start_date, end_date, longitude, latitude, address

    Returns:
        bool: valide or not valide
    """
    try:
        allowed_raduis = [0.99, 5, 10, 15, 20, 25, 30]
        if search_by_department:
            allowed_raduis.append(100)

        valide = (
            float(search_criteria["radius_km"]) in allowed_raduis
            and bool(datetime.strptime(search_criteria["start_date"], "%Y-%m-%d"))
            and bool(datetime.strptime(search_criteria["end_date"], "%Y-%m-%d"))
            and isfloat(float(search_criteria["longitude"]))
            and isfloat(float(search_criteria["latitude"]))
        )
    except ValueError:
        valide = False
    return valide


def capitalize_custom(name: str) -> str:
    """Capitalize the first letter of every word with some exceptions

    Args:
        name : str

    Returns:
        name : str
    """
    result = name.title()
    lower_words = ["Sur", "La", "Le", "Les", "Lès", "De", "Des", "En", "Et", "Aux"]
    for lower_word in lower_words:
        result = (
            result.replace("-" + lower_word + "-", "-" + lower_word.lower() + "-")
            .replace(" " + lower_word + "-", " " + lower_word.lower() + "-")
            .replace(" " + lower_word + " ", " " + lower_word.lower() + " ")
        )

    result = (
        result.replace(" L'", " l'")
        .replace(" D'", " d'")
        .replace("-L'", "-l'")
        .replace("-D'", "-d'")
    )

    return result


def is_date_after_paris_datetime(date: datetime) -> bool:
    """Check if the date is after the current date in Paris timezone

    Args:
        date : datetime

    Returns:
        bool
    """
    utc_time = datetime.utcnow()
    paris_offset = timedelta(hours=1)
    paris_time = utc_time + paris_offset
    return date < paris_time
